/******************************************************************************
* NCSimHelp.cpp - holds definitions of all help/tooltips in NCSim.	      *
*******************************************************************************
* (C) Gershon Elber					                      *
*******************************************************************************
* Written by Gershon Elber, Dec 2006.					      *
******************************************************************************/

#include "NCSimMain.h"

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Given a widget ID, returns its tool tip as a string.                     M
*                                                                            *
* PARAMETERS:                                                                M
*   Id:       Id assigned to widget.                                         M
*                                                                            *
* RETURN VALUE:                                                              M
*   const wxString:  Tool tip description.                                   M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimToolTip		                                             M
*****************************************************************************/
const wxString NCSimToolTip(int Id)
{
    const char
	*RetVal = "";

    switch (Id) {
        /* Tool tips of animation controls. */
	case wxID_NCSIM_ANIM_FBWRD:
	    RetVal = "Halfs forward animation speed\nDoubles backward animation speed";
	    break;
	case wxID_NCSIM_ANIM_BWRD:
	    RetVal = "Backward animation";
	    break;
	case wxID_NCSIM_ANIM_BWRD_STEP:
	    RetVal = "Backward single step animation";
	    break;
	case wxID_NCSIM_ANIM_STOP:
	    RetVal = "Stop the animation completely";
	    break;
	case wxID_NCSIM_ANIM_PAUSE:
	    RetVal = "Pause the animation";
	    break;
	case wxID_NCSIM_ANIM_FWRD_STEP:
	    RetVal = "Forward single step animation";
	    break;
	case wxID_NCSIM_ANIM_FWRD:
	    RetVal = "Forward animation";
	    break;
	case wxID_NCSIM_ANIM_FFWRD:
	    RetVal = "Doubles forward animation speed\nHalfs backward animation speed";
	    break;
	case wxID_NCSIM_ANIM_SLIDER:
	    RetVal = "Current tool animation location control";
	    break;
	case wxID_NCSIM_ANIM_LINE_NUM:
	    RetVal = "Presents NC file line number\nof currently executed G-code";
	    break;
	case wxID_NCSIM_ANIM_NC_TEXT:
	    RetVal = "Currently executed NC G-code line";
	    break;

	/* Tool tips of simulation parameters widget. */
	case wxID_NCSIM_PARAM_REL_ARC_CNTR:
	    RetVal = "Check for arc centers relative to initial arc position\nUncheck for absolute arc center positions";
	    break;
	case wxID_NCSIM_PARAM_REVERSE_Z_DIR:
	    RetVal = "Check to reverse the Z direction so\npositive Z is down (away from machining tool)";
	    break;
	case wxID_NCSIM_PARAM_TOOL_NUM:
	    RetVal = "The tool index to use in the simulation";
	    break;
	case wxID_NCSIM_PARAM_UNITS:
	    RetVal = "Units to employ in the simulation:\nmilimeters or inches";
	    break;
	case wxID_NCSIM_PARAM_MAX_MRR:
	    RetVal = "Sets the maximal material removal rate (MRR) to allow,\nin volume per unit-time";
	    break;
	case wxID_NCSIM_PARAM_TOOL_TYPE:
	    RetVal = "Sets the tool type:\nball end, flat end, etc.";
	    break;
	case wxID_NCSIM_PARAM_ZBUF_RES:
	    RetVal = "Sets the resolution of the Z buffer,\nin pixels, in the larger dimension";
	    break;
	case wxID_NCSIM_PARAM_NC_Z_BUF:
	    RetVal = "Check to activate the Z buffer NC simulation\nUncheck to disable the Z Buffer (to only animate the tool motion)";
	    break;
	case wxID_NCSIM_PARAM_PAUSE_ERROR:
	    RetVal = "Check to pause the simulation when errors are detected in NC-codes\nUncheck to continue (warnings messages are logged either way)";
	    break;
	case wxID_NCSIM_PARAM_UPDATE_FEEDRATES:
	    RetVal = "Check to update feedrates when the material removal is too fast.\nIn order to save these changes you will have to save simulation into a new NC files.";
	    break;
	case wxID_NCSIM_PARAM_RESET_FEEDRATES:
	    RetVal = "Press to reset feedrates to original input file values.";
	    break;
	case wxID_NCSIM_PARAM_STOCK_GEOM_INIT:
	    RetVal = "Allows the initialization of a general shaped stock over the stock bbox.\nOnly takes effect in simulation initialization";
	    break;
	case wxID_NCSIM_PARAM_SIM_MODE:
	    RetVal = "Real time for simulation following the actual time\nFixed step for a fixed motion step in each iteration";
	    break;
	case wxID_NCSIM_PARAM_SIM_STEP:
	    RetVal = "Sets the tool step size/time step to take in cutting motion\nwhen a fixed/time step simulation is selected";
	    break;
	case wxID_NCSIM_PARAM_FEED_RATE:
	    RetVal = "Sets the default feed rate to move the tool in.\nOnly takes effect on NC file load";
	    break;
	case wxID_NCSIM_PARAM_SPINDLE_SP:
	    RetVal = "Sets the spindle speed of the tool.\nOnly takes effect on NC file load";
	    break;
	case wxID_NCSIM_PARAM_FAST_SPEED_FACTOR:
	    RetVal = "Multiplicative factor to use in the simulation to speed\nup fast, G0, motions with respect to cutting motion";
	    break;
	case wxID_NCSIM_PARAM_GEOM_APPROX_TOL:
	    RetVal = "Geometry approximation (to polygons) tolerance";
	    break;
	case wxID_NCSIM_PARAM_TOOL_DIAM:
	    RetVal = "Sets the diameter of the tool to use";
	    break;
	case wxID_NCSIM_PARAM_TOOL_TORUS_RAD:
	    RetVal = "Sets the torus radius of a round torus/bull-nose tool";
	    break;
	case wxID_NCSIM_PARAM_TOOL_ICOLOR:
	    RetVal = "Sets the tools' color in idle time";
	    break;
	case wxID_NCSIM_PARAM_TOOL_FCOLOR:
	    RetVal = "Sets the tools' color in fast, G0, motion time";
	    break;
	case wxID_NCSIM_PARAM_TOOL_CCOLOR:
	    RetVal = "Sets the tools' color in cutting motion time";
	    break;
	case wxID_NCSIM_PARAM_STOCK_COLOR:
	    RetVal = "Sets the color of the stock";
	    break;
	case wxID_NCSIM_PARAM_STOCK_TRANSP:
	    RetVal = "Sets the transparecy display level of\nthe stock, between zero and one";
	    break;
	case wxID_NCSIM_PARAM_STOCK_BBOX:
	    RetVal = "Sets the bounding box dimensions of the stock";
	    break;

	/*  Tool tips of  of rendering (OGL) parameters widget. */
	case wxID_NCSIM_OGL_PARAM_ORTHO_PERSP:
	    RetVal = "Selects between orthographic projection\nand persective projection display";
	    break;
	case wxID_NCSIM_OGL_PARAM_SOLID_WIRE:
	    RetVal = "Selects between solid (shaded) display\nand wireframe based display";
	    break;
	case wxID_NCSIM_OGL_PARAM_LINE_WIDTH:
	    RetVal = "Controls the line width, in pixels,\nin which the drawings are using";
	    break;
	case wxID_NCSIM_OGL_PARAM_FRAME_SKIP:
	    RetVal = "How many simulation steps should be taken before the display is refreshed.\nUse to speed up the total simulation time";
	    break;
	case wxID_NCSIM_OGL_PARAM_MOUSE_SENS:
	    RetVal = "Controls the mouse senstivity control over the\nrotation, scale, and translation of the geometry";
	    break;
	case wxID_NCSIM_OGL_PARAM_AXES_SIZE:
	    RetVal = "Controls the length of the drawn axes";
	    break;
	case wxID_NCSIM_OGL_PARAM_LIGHT0POS:
	    RetVal = "Position the 1st light source\n(w = 0 for a light source at infinity)";
	    break;
	case wxID_NCSIM_OGL_PARAM_LIGHT0CLR:
	    RetVal = "Color of 1st light source";
	    break;
	case wxID_NCSIM_OGL_PARAM_LIGHT1POS:
	    RetVal = "Position the 2nd light source\n(w = 0 for a light source at infinity)";
	    break;
	case wxID_NCSIM_OGL_PARAM_LIGHT1CLR:
	    RetVal = "Color of 2st light source";
	    break;
	case wxID_NCSIM_OGL_PARAM_BG_COLOR:
	    RetVal = "Sets the color of the background";
	    break;
	case wxID_NCSIM_OGL_PARAM_DRAW_AXES:
	    RetVal = "Check to draw axes\nUnceck to hide the axes";
	    break;
	case wxID_NCSIM_OGL_PARAM_DRAW_TOOL:
	    RetVal = "Check to draw the tool\nUncheck to hide it";
	    break;
	case wxID_NCSIM_OGL_PARAM_DRAW_TPATH:
	    RetVal = "Check to draw the tool path\nUncheck to hide it";
	    break;
	case wxID_NCSIM_OGL_PARAM_DRAW_STOCK:
	    RetVal = "Check to draw the stock\nUncheck to hide it";
	    break;
	case wxID_NCSIM_OGL_PARAM_DRAW_FINAL_PART:
	    RetVal = "Check to draw the final part\n(if available) Uncheck to hide it";
	    break;
	case wxID_NCSIM_OGL_PARAM_DRAW_OTHER_GEOM:
	    RetVal = "Check to draw other geometry\n(if available) Uncheck to hide it";
	    break;
	case wxID_NCSIM_OGL_PARAM_MOUSE_SENSITIVITY:
	    RetVal = "Multiplicative control factor over\nmouse sensitivity transformations";
	    break;
    }

    assert(RetVal != "");

    return NCSimChars2wxString(RetVal);
}
