/******************************************************************************
* NCSimMain.cpp - Main entry point of NCSim.				      *
*******************************************************************************
* (C) Gershon Elber					                      *
*******************************************************************************
* Written by Gershon Elber, Dec 2006.					      *
******************************************************************************/

#include "irit_sm.h"
#include "misc_lib.h"
#include "grap_lib.h"
#include "iritprsr.h"

#include "NCSimMain.h"
#include "NCSimOGL.h"
#include "NCSimMessages.h"
#include "NCSimulation.h"
#include "Resources/NCSimLogo.xpm"
#include "Resources/NCSimSplash.xpm"
#include "Resources/NCSimIconBwrdStep.xpm"
#include "Resources/NCSimIconBwrd.xpm"
#include "Resources/NCSimIconFBwrd.xpm"
#include "Resources/NCSimIconStop.xpm"
#include "Resources/NCSimIconPause.xpm"
#include "Resources/NCSimIconFrwdStep.xpm"
#include "Resources/NCSimIconFrwd.xpm"
#include "Resources/NCSimIconFFrwd.xpm"

#include <wx/stdpaths.h>
#include <wx/splash.h>
#include <wx/aboutdlg.h>
#include <wx/wupdlock.h>

IMPLEMENT_APP(NCSimApp) /* Give wxWidgets means to create a NCSimApp object. */

IRIT_STATIC_DATA char
    *GlblBackGroundStr = "0, 0, 0",
    *GlblGuiStatusFName = "NCSimGui.cfg",
    *GlblLightSrc1PosStr = "1.0,  2.0,  5.0, 0.0",
    *GlblLightSrc2PosStr = "1.0, -1.0, -3.0, 0.0",
    *GlblToolGeomFName = NULL,
    *GlblStockGeomFName = NULL,
    *GlblPartGeomFName = NULL,
    *GlblNCFileName = NULL;

IRIT_STATIC_DATA int
    GlblFullRcntFName = TRUE,
    GlblRichTextCtrlNCFile = FALSE,
    GlblZBufferSize = 1000,
    GlblNCZBufSaveITD = FALSE,
    GlblUseOGLDispLists = FALSE;

IRIT_STATIC_DATA FILE
    *GlblNCZBufSaveFile = NULL;

IRIT_STATIC_DATA NCSimUnitsType
    GlblUnits = NCSIM_UNIT_MM;

IRIT_STATIC_DATA NCSimToolType
    GlblToolType = NCSIM_TOOL_TYPE_BALL_END;

IRIT_STATIC_DATA IrtRType
    GlblToolDiameter = 2.5;

IRIT_STATIC_DATA GMBBBboxStruct
    GlblStockBBox = { { 0.0, 0.0, 0.0 },
		      { 0.0, 0.0, 0.0 } };

IRIT_STATIC_DATA IritConfigStruct NCSimSetUp[] =
{
  { "BackGround",     "-b", (VoidPtr) &GlblBackGroundStr,    IC_STRING_TYPE },
  { "LightSrcPos",    "-L", (VoidPtr) &GlblLightSrc1PosStr,  IC_STRING_TYPE },
  { "LightSrcPos2",   "-L", (VoidPtr) &GlblLightSrc2PosStr,  IC_STRING_TYPE },
  { "ToolGeometry",   "-t", (VoidPtr) &GlblToolGeomFName,    IC_STRING_TYPE },
  { "StockGeometry",  "-s", (VoidPtr) &GlblStockGeomFName,   IC_STRING_TYPE },
  { "PartGeometry",   "-p", (VoidPtr) &GlblPartGeomFName,    IC_STRING_TYPE },
  { "NCFileName",     "-g", (VoidPtr) &GlblNCFileName,       IC_STRING_TYPE },
  { "NCSimGuiCfg",    "",   (VoidPtr) &GlblGuiStatusFName,   IC_STRING_TYPE },
  { "DrawVNormal",    "-n", (VoidPtr) &IGGlblDrawVNormal,    IC_BOOLEAN_TYPE },
  { "DrawPNormal",    "-N", (VoidPtr) &IGGlblDrawPNormal,    IC_BOOLEAN_TYPE },
  { "MoreVerbose",    "-m", (VoidPtr) &IGGlblMore,	     IC_BOOLEAN_TYPE },
  { "DrawStyle",      "-r", (VoidPtr) &IGGlblDrawStyle,	     IC_BOOLEAN_TYPE },
  { "BFaceCull",      "-B", (VoidPtr) &IGGlblBackFaceCull,   IC_BOOLEAN_TYPE },
  { "DoubleBuffer",   "-2", (VoidPtr) &IGGlblDoDoubleBuffer, IC_BOOLEAN_TYPE },
  { "DepthCue",	      "-c", (VoidPtr) &IGGlblDepthCue,	     IC_BOOLEAN_TYPE },
  { "FourPerFlat",    "-4", (VoidPtr) &IGGlblFourPerFlat,    IC_BOOLEAN_TYPE },
  { "AntiAlias",      "-a", (VoidPtr) &IGGlblAntiAliasing,   IC_BOOLEAN_TYPE },
  { "DrawSurfaceMesh","-M", (VoidPtr) &IGGlblDrawSurfaceMesh,IC_BOOLEAN_TYPE },
  { "DrawSurfacePoly","-P", (VoidPtr) &IGGlblDrawSurfacePoly,IC_BOOLEAN_TYPE },
  { "DrawSurfaceWire","-W", (VoidPtr) &IGGlblDrawSurfaceWire,IC_BOOLEAN_TYPE },
  { "ContMotion",     "-C", (VoidPtr) &IGGlblContinuousMotion,IC_BOOLEAN_TYPE },
  { "NrmlOrientation","-o", (VoidPtr) &IGGlblFlipNormalOrient,IC_BOOLEAN_TYPE },
  { "OGLDispLists",   "-D", (VoidPtr) &GlblUseOGLDispLists,  IC_BOOLEAN_TYPE },
  { "FullRcntFName",  "",   (VoidPtr) &GlblFullRcntFName,    IC_BOOLEAN_TYPE },
  { "RichText",       "",   (VoidPtr) &GlblRichTextCtrlNCFile,IC_BOOLEAN_TYPE },
  { "NumOfIsolines",  "-I", (VoidPtr) &IGGlblNumOfIsolines,  IC_INTEGER_TYPE },
  { "LineWidth",      "-l", (VoidPtr) &IGGlblLineWidth,	     IC_INTEGER_TYPE },
  { "PolygonOpti",    "-F", (VoidPtr) &IGGlblPolygonOptiApprox,IC_INTEGER_TYPE },
  { "PolylineOpti",   "-f", (VoidPtr) &IGGlblPolylineOptiApprox,IC_INTEGER_TYPE },
  { "ShadingModel",   "-A", (VoidPtr) &IGGlblShadingModel,   IC_INTEGER_TYPE },
  { "TransMode",      "",   (VoidPtr) &IGGlblTransformMode,  IC_INTEGER_TYPE },
  { "ViewMode",	      "",   (VoidPtr) &IGGlblViewMode,	     IC_INTEGER_TYPE },
  { "ZBufSize",       "-z", (VoidPtr) &GlblZBufferSize,      IC_INTEGER_TYPE },
  { "ToolType",       "-T", (VoidPtr) &GlblToolType,         IC_INTEGER_TYPE },
  { "Units",          "-u", (VoidPtr) &GlblUnits,            IC_INTEGER_TYPE },
  { "ZClipMin",	      "-Z", (VoidPtr) &IGGlblZMinClip,	     IC_REAL_TYPE },
  { "ZClipMax",	      "-Z", (VoidPtr) &IGGlblZMaxClip,	     IC_REAL_TYPE },
  { "PlgnFineNess",   "-F", (VoidPtr) &IGGlblPlgnFineness,   IC_REAL_TYPE },
  { "PllnFineNess",   "-f", (VoidPtr) &IGGlblPllnFineness,   IC_REAL_TYPE },
  { "PointSize",      "-w", (VoidPtr) &IGGlblPointSize,	     IC_REAL_TYPE },
  { "RelLowRes",      "-E", (VoidPtr) &IGGlblRelLowresFineNess,IC_REAL_TYPE },
  { "NormalSize",     "-R", (VoidPtr) &IGGlblNormalSize,     IC_REAL_TYPE },
  { "ToolDiameter",   "-T", (VoidPtr) &GlblToolDiameter,     IC_REAL_TYPE },
  { "StockMinX",      "-S", (VoidPtr) &GlblStockBBox.Min[0], IC_REAL_TYPE },
  { "StockMinY",      "-S", (VoidPtr) &GlblStockBBox.Min[1], IC_REAL_TYPE },
  { "StockMinZ",      "-S", (VoidPtr) &GlblStockBBox.Min[2], IC_REAL_TYPE },
  { "StockMaxX",      "-S", (VoidPtr) &GlblStockBBox.Max[0], IC_REAL_TYPE },
  { "StockMaxY",      "-S", (VoidPtr) &GlblStockBBox.Max[1], IC_REAL_TYPE },
  { "StockMaxZ",      "-S", (VoidPtr) &GlblStockBBox.Max[2], IC_REAL_TYPE }
};
IRIT_STATIC_DATA const int 
    NUM_SET_UP = (sizeof(NCSimSetUp) / sizeof(IritConfigStruct));

IRIT_STATIC_DATA const IritConfigStruct NCSimGUICfgSetUp[] = {
	{ "MainWinPos",      "1", (VoidPtr) NULL,    IC_STRING_TYPE },
	{ "MainWinSize",     "2", (VoidPtr) NULL,    IC_STRING_TYPE },
	{ "WinState",        "3", (VoidPtr) NULL,    IC_STRING_TYPE },
	{ "RcntFileNames",   "4", (VoidPtr) NULL,    IC_MULTI_STR_TYPE }
    };
IRIT_STATIC_DATA const int
    NCSIM_GUI_CFG_NUM_SET_UP =
		       sizeof(NCSimGUICfgSetUp) / sizeof(IritConfigStruct);

IRIT_STATIC_DATA char
    *GenCtrlStr = " n%- N%- i%- c%- m%- a%- I%-#IsoLines!d F%-PlgnOpti|PlgnFineNess!d!F f%-PllnOpti|PllnFineNess!d!F E%-RelLowRes!F w%-Point|Size!F l%-Line|Width!d r%- A%-Shader!d B%- 2%- R%-Normal|Size!F 4%- b%-\"R,B,G|(background)\"!s L%-\"x,y,z,w{,a,d,s}\"!s Z%-ZClipMin|ZClipMax!F!F M%- W%-WireSetup!d P%- o%- C%- t%-ToolGeometry!s s%-StockGeometry!s p%-PartGeometry!s g%-GCodeFile!s z%-NCZBufSize!d S%-StockMinX|Y|Z|MaxX|Y|Z!F!F!F!F!F!F T%-ToolType|ToolDiam!d%F u%-Units!d D%- DFiles!*s";

IRIT_GLOBAL_DATA NCSimMainFrame *NCSimMF;

static void UserNCGCodeParserError(char *Line);
static void RedirectIOToConsole(void);
static void SaveOneNCZBufferTriangle(IrtRType X1,
				     IrtRType Y1,
				     IrtRType Z1,
				     IrtRType X2,
				     IrtRType Y2,
				     IrtRType Z2,
				     IrtRType X3,
				     IrtRType Y3,
				     IrtRType Z3);

/*****************************************************************************
* AUXILIARY:								     *
*    Event table for NCSimMainFrame/NCSimSplitMainFrame			     *
*****************************************************************************/
BEGIN_EVENT_TABLE(NCSimMainFrame, wxFrame)
    EVT_MENU(wxID_NCSIM_FILE_MENU_IMPORT_GEOM,          ImportGeometry)
    EVT_MENU(wxID_NCSIM_FILE_MENU_IMPORT_FINAL_PART,    ImportFinalPart)
    EVT_MENU(wxID_NCSIM_FILE_MENU_LOAD_NCFILE,		LoadNCFile)
    EVT_MENU(wxID_NCSIM_FILE_MENU_SAVE_NCFILE,		SaveNCFile)
    EVT_MENU(wxID_NCSIM_FILE_MENU_CLOSE_NCFILE,		CloseNCFile)
    EVT_MENU(wxID_NCSIM_FILE_MENU_EXPORT_TPATH,		SaveNCTPathToFile)
    EVT_MENU(wxID_NCSIM_FILE_MENU_SAVE_NCZBUF,		SaveNCZBufToFile)
    EVT_MENU(wxID_NCSIM_FILE_MENU_EXIT,			OnQuit)
    EVT_MENU(wxID_NCSIM_FILE_MENU_RECENT_FNAME,		OnRecentFNames)
    EVT_MENU(wxID_NCSIM_FILE_MENU_RECENT_FNAME+1,	OnRecentFNames)
    EVT_MENU(wxID_NCSIM_FILE_MENU_RECENT_FNAME+2,	OnRecentFNames)
    EVT_MENU(wxID_NCSIM_FILE_MENU_RECENT_FNAME+3,	OnRecentFNames)
    EVT_MENU(wxID_NCSIM_FILE_MENU_RECENT_FNAME+4,	OnRecentFNames)

    EVT_MENU(wxID_NCSIM_STATE_MENU_SET_ALL_PARAMS,	StateViewSimMode)
    EVT_MENU(wxID_NCSIM_STATE_MENU_SET_OGL_PARAMS,	StateViewSimMode)
    EVT_MENU(wxID_NCSIM_VIEW_MENU_VIEW_UP,		StateViewSimMode)
    EVT_MENU(wxID_NCSIM_VIEW_MENU_VIEW_FRONT,		StateViewSimMode)
    EVT_MENU(wxID_NCSIM_VIEW_MENU_VIEW_SIDE,		StateViewSimMode)
    EVT_MENU(wxID_NCSIM_VIEW_MENU_RESCALE_VIEW,		StateViewSimMode)

    EVT_MENU(wxID_NCSIM_HELP_MENU_ABOUT,		OnAbout)

    EVT_MENU(wxID_NCSIM_ANIM_FBWRD,			AnimControl)
    EVT_MENU(wxID_NCSIM_ANIM_BWRD,			AnimControl)
    EVT_MENU(wxID_NCSIM_ANIM_BWRD_STEP,			AnimControl)
    EVT_MENU(wxID_NCSIM_ANIM_STOP,			AnimControl)
    EVT_MENU(wxID_NCSIM_ANIM_PAUSE,			AnimControl)
    EVT_MENU(wxID_NCSIM_ANIM_FWRD_STEP,			AnimControl)
    EVT_MENU(wxID_NCSIM_ANIM_FWRD,			AnimControl)
    EVT_MENU(wxID_NCSIM_ANIM_FFWRD,			AnimControl)
    EVT_MENU_HIGHLIGHT(wxID_ANY,			ShowMenuHelp)
    EVT_SLIDER(wxID_NCSIM_ANIM_SLIDER,			AnimSliderControl)
    EVT_TEXT(wxID_NCSIM_ANIM_LINE_NUM,			OnTextChange)

    EVT_AUINOTEBOOK_PAGE_CHANGED(wxID_ANY,		NCSimPageChanged)
    EVT_AUINOTEBOOK_PAGE_CHANGING(wxID_ANY,		NCSimPageChanged)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(NCSimSplitMainFrame, wxSplitterWindow)
END_EVENT_TABLE()

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Initialize the application.                                              M
*                                                                            *
* PARAMETERS:                                                                M
*   None                                                                     M
*                                                                            *
* RETURN VALUE:                                                              M
*   bool:     TRUE to start the Event loop.                                  M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimApp::OnInit                                                         M
*****************************************************************************/
bool NCSimApp::OnInit()
{
    NCSIM_DEBUG_FUNC_ENTRY

    int i, Error,
	NumFiles = 0,
	DrawSrfWireSetup = 0,
	DrawSrfWireFlag = FALSE,
	TransPosFlag = FALSE,
	ViewPosFlag = FALSE,
	IsoLinesFlag = FALSE,
	SrfFinenessFlag = FALSE,
	CrvFinenessFlag = FALSE,
	LineWidthFlag = FALSE,
	NormalSizeFlag = FALSE,
	BackGroundFlag = FALSE,
	LightSrcPosFlag = FALSE,
	MinPickDistFlag = FALSE,
	PickObjTypeFlag = FALSE,
	ZClipPlaneFlag = FALSE,
	PointSizeFlag = FALSE,
	ShadingModelFlag = FALSE,
	SketchStyleFlag = FALSE,
	ExecAnimEachStepFlag = FALSE,
	ExecAnimationFlag = FALSE,
	LowResFineNessFlag = FALSE,
	InitWidgetFlag = FALSE,
	AntiAliasingFlag = FALSE,
	DrawSolidFlag = FALSE,
	ToolGeometryFileFlag = FALSE,
	StockGeometryFileFlag = FALSE,
	PartGeometryFileFlag = FALSE,
	NCGCodeFileFlag = FALSE,
	NCZBufSizeFlag = FALSE,
	StockSizeFlag = FALSE,
	ToolTypeFlag = FALSE,
	WorkingUnitsFlag = FALSE;
    char Line[IRIT_LINE_LEN_VLONG], **FileNames, *UName;
    const char
	**ArgvChar = new const char *[argc];
    float Ambient, Diffuse, Specular;

#   ifdef _DEBUG
        RedirectIOToConsole();
	IritInitTestDynMemory();
#   else
    {
	wxImage
	    Image(NCSimSplash);
	wxSplashScreen
	    *splash = new wxSplashScreen(Image,
			wxSPLASH_CENTRE_ON_SCREEN | wxSPLASH_TIMEOUT,
			2000, NULL, -1, wxDefaultPosition, wxDefaultSize,
			wxSIMPLE_BORDER | wxSTAY_ON_TOP);
	wxYield();

	if (IGGlblMore)
	    RedirectIOToConsole();
    }
#   endif /* _DEBUG */

    /* Initialize the default global display setup. */
    IGGlblDrawVNormal = FALSE;
    IGGlblDrawPNormal = FALSE;
    IGGlblMore = FALSE;
    IGGlblDrawStyle = TRUE;
    IGGlblBackFaceCull = FALSE;
    IGGlblDoDoubleBuffer = TRUE;
    IGGlblDepthCue = FALSE;
    IGGlblFourPerFlat = FALSE;
    IGGlblAntiAliasing = FALSE;
    IGGlblDrawSurfaceMesh = FALSE;
    IGGlblDrawSurfacePoly = TRUE;
    IGGlblDrawSurfaceWire = FALSE;
    IGGlblContinuousMotion = FALSE;
    IGGlblFlipNormalOrient = FALSE;
    IGGlblNumOfIsolines = 10;
    IGGlblLineWidth = 1;
    IGGlblPolygonOptiApprox = 1;
    IGGlblPolylineOptiApprox = SYMB_CRV_APPROX_TOLERANCE;
    IGGlblShadingModel = 4;
    IGGlblTransformMode = 1;
    IGGlblViewMode = 2;
    IGGlblZMinClip = -100.0;
    IGGlblZMaxClip = 100.0;
    IGGlblPlgnFineness = 0.005;
    IGGlblPllnFineness = 0.005;
    IGGlblPointSize = 0.02;
    IGGlblRelLowresFineNess = 0.3;
    IGGlblNormalSize = 0.1;
    IGGlblChangeFactor = 1.0;

    for (i = 0; i < argc; i++) {
	ArgvChar[i] = IritStrdup(NCSimwxString2Chars(argv[i]));
    }

    /* Read config. file. */
    wxStandardPathsBase
        &StdPaths = wxStandardPaths::Get();

    if (IritConfig(NCSimwxString2Chars(StdPaths.GetExecutablePath()),
		   NCSimSetUp, NUM_SET_UP))
        NCSimPrintf(NCSIM_LOG_INFO, "Loaded config file for \"%s\".\n",
		    NCSimwxString2Chars(StdPaths.GetExecutablePath()));
    else
        NCSimPrintf(NCSIM_LOG_ERROR, "Failed to load config file for \"%s\".\n",
		    NCSimwxString2Chars(StdPaths.GetExecutablePath()));

    strcpy(Line, ArgvChar[0]);
    strcat(Line, GenCtrlStr);

    if ((Error = GAGetArgs(argc, ArgvChar, Line, 
			   /* Graphics command line Options:*/
			   &IGGlblDrawVNormal, &IGGlblDrawPNormal,
			   &IGGlblDrawInternal,
			   &IGGlblDepthCue, &IGGlblMore,
			   &IGGlblAntiAliasing,
			   &IsoLinesFlag, &IGGlblNumOfIsolines,
			   &SrfFinenessFlag, &IGGlblPolygonOptiApprox,
			   &IGGlblPlgnFineness, 
			   &CrvFinenessFlag, &IGGlblPolylineOptiApprox,
			   &IGGlblPllnFineness, 
			   &LowResFineNessFlag, &IGGlblRelLowresFineNess,
			   &PointSizeFlag, &IGGlblPointSize,
			   &LineWidthFlag, &IGGlblLineWidth,
			   &DrawSolidFlag,
			   &ShadingModelFlag, &IGGlblShadingModel,
			   &IGGlblBackFaceCull, &IGGlblDoDoubleBuffer,
			   &NormalSizeFlag, &IGGlblNormalSize,
			   &IGGlblFourPerFlat,
			   &BackGroundFlag, &GlblBackGroundStr,
			   &LightSrcPosFlag, &GlblLightSrc1PosStr,
			   &ZClipPlaneFlag, &IGGlblZMinClip, &IGGlblZMaxClip,
			   &IGGlblDrawSurfaceMesh,
			   &DrawSrfWireFlag, &DrawSrfWireSetup,
			   &IGGlblDrawSurfacePoly,
			   &IGGlblFlipNormalOrient, &IGGlblContinuousMotion,
			   /* NCSim command line Options: */
			   &ToolGeometryFileFlag, &GlblToolGeomFName,
			   &StockGeometryFileFlag, &GlblStockGeomFName,
			   &PartGeometryFileFlag, &GlblPartGeomFName,
			   &NCGCodeFileFlag, &GlblNCFileName,
			   &NCZBufSizeFlag, &GlblZBufferSize,
			   &StockSizeFlag, &GlblStockBBox.Min[0],
					   &GlblStockBBox.Min[1],
					   &GlblStockBBox.Min[2],
					   &GlblStockBBox.Max[0],
					   &GlblStockBBox.Max[1],
					   &GlblStockBBox.Max[2],
			   &ToolTypeFlag, &GlblToolType,
					  &GlblToolDiameter,
			   &WorkingUnitsFlag, &GlblUnits,
			   &GlblUseOGLDispLists,
			   &NumFiles, &FileNames)) != 0) {
        GAStringErrMsg(Error, Line);
	NCSimPrintf(NCSIM_LOG_ERROR, "%s.\n", Line);
	GAStringHowTo(GenCtrlStr, Line);
	NCSimPrintf(NCSIM_LOG_ERROR, "%s.\n", Line);
    }

    if (IGGlblDrawStyle == 0)
	IGGlblDrawStyle = IG_STATE_DRAW_STYLE_WIREFRAME;
    else
        IGGlblDrawStyle = IG_STATE_DRAW_STYLE_SOLID;
    if (DrawSolidFlag)
	IGGlblDrawStyle = IG_STATE_DRAW_STYLE_SOLID;

    if (IGGlblAntiAliasing == 0)
	IGGlblAntiAliasing = IG_STATE_ANTI_ALIAS_OFF;
    else
        IGGlblAntiAliasing = IG_STATE_ANTI_ALIAS_ON;
    if (AntiAliasingFlag)
	IGGlblAntiAliasing = IG_STATE_ANTI_ALIAS_ON;

    if (sscanf(GlblBackGroundStr, "%d %d %d",
	       &IGGlblBackGroundColor[0],
	       &IGGlblBackGroundColor[1],
	       &IGGlblBackGroundColor[2]) != 3)
        sscanf(GlblBackGroundStr, "%d,%d,%d",
	       &IGGlblBackGroundColor[0],
	       &IGGlblBackGroundColor[1],
	       &IGGlblBackGroundColor[2]);

    if (sscanf(GlblLightSrc1PosStr, "%f %f %f %f %f %f %f",
	       &IGShadeParam.LightPos[0][0],
	       &IGShadeParam.LightPos[0][1],
	       &IGShadeParam.LightPos[0][2],
	       &IGShadeParam.LightPos[0][3],
	       &Ambient, &Diffuse, &Specular) == 7 ||
	sscanf(GlblLightSrc1PosStr, "%f,%f,%f,%f,%f,%f,%f",
	       &IGShadeParam.LightPos[0][0],
	       &IGShadeParam.LightPos[0][1],
	       &IGShadeParam.LightPos[0][2],
	       &IGShadeParam.LightPos[0][3],
	       &Ambient, &Diffuse, &Specular) == 7) {
	/* Need to update intensities as well. */
        IGShadeParam.LightAmbient[0] =
	    IGShadeParam.LightAmbient[1] =
		IGShadeParam.LightAmbient[2] = Ambient;
	IGShadeParam.LightDiffuse[0] =
		IGShadeParam.LightDiffuse[1] =
		    IGShadeParam.LightDiffuse[2] = Diffuse;
	IGShadeParam.LightSpecular[0] =
		IGShadeParam.LightSpecular[1] =
		    IGShadeParam.LightSpecular[2] = Specular;
    }
    else {
        if (sscanf(GlblLightSrc1PosStr, "%f %f %f %f",
		   &IGShadeParam.LightPos[0][0],
		   &IGShadeParam.LightPos[0][1],
		   &IGShadeParam.LightPos[0][2],
		   &IGShadeParam.LightPos[0][3]) != 4)
	  sscanf(GlblLightSrc1PosStr, "%f,%f,%f,%f",
		 &IGShadeParam.LightPos[0][0],
		 &IGShadeParam.LightPos[0][1],
		 &IGShadeParam.LightPos[0][2],
		 &IGShadeParam.LightPos[0][3]);
    }

    if (sscanf(GlblLightSrc2PosStr, "%f %f %f %f",
	       &IGShadeParam.LightPos[1][0],
	       &IGShadeParam.LightPos[1][1],
	       &IGShadeParam.LightPos[1][2],
	       &IGShadeParam.LightPos[1][3]) != 4)
        sscanf(GlblLightSrc2PosStr, "%f,%f,%f,%f",
	       &IGShadeParam.LightPos[1][0],
	       &IGShadeParam.LightPos[1][1],
	       &IGShadeParam.LightPos[1][2],
	       &IGShadeParam.LightPos[1][3]);

    if (ShadingModelFlag &&
	(IGGlblShadingModel < IG_SHADING_NONE ||
	 IGGlblShadingModel > IG_SHADING_PHONG)) {
        NCSimPrintf(NCSIM_LOG_ERROR,
		    "Shading Model between 0 (None) and 3 (Phong) only.\n");
	GAStringHowTo(GenCtrlStr, Line);
	NCSimPrintf(NCSIM_LOG_ERROR, Line);
    }

    if (DrawSrfWireFlag) {
        IGGlblDrawSurfaceWire = DrawSrfWireSetup & 0x01;
        IGGlblDrawSurfaceBndry = DrawSrfWireSetup & 0x02;
        IGGlblDrawSurfaceSilh = DrawSrfWireSetup & 0x04;
        IGGlblDrawSurfaceSketch = DrawSrfWireSetup & 0x08;
        IGGlblDrawSurfaceRflctLns = DrawSrfWireSetup & 0x10;
    }

    /* Update the dynamic config file name with the USERNAME. */
    if (strstr(GlblGuiStatusFName, "USERNAME") != NULL &&
	(UName = getenv("USERNAME")) != NULL) {
        char *p, Line2[IRIT_LINE_LEN_VLONG];

        /* Replacing USERNAME with real username first. */
	strcpy(Line, GlblGuiStatusFName);
	p = strstr(Line, "USERNAME");
	*p = 0;

	sprintf(Line2, "%s%s%s", Line, UName, &p[8]);
	GlblGuiStatusFName = IritStrdup(Line2);
    }

    /* Create the main application window, and show it. */
    NCSimMF = new NCSimMainFrame(wxT("NCSim"), wxSize(800, 800));

    NCSimPrintf(NCSIM_LOG_INFO, "Welcome to NCSim Ver. " NCSIM_VERSION "\n");

    NCSimMF -> Show(true);

    /* Upload the files prescribed in the command line. */
    if (GlblNCFileName == NULL ||
	strlen(GlblNCFileName) == 0 ||
	!NCSimMF -> CreateNewSim(GlblNCFileName)) {
        /* Create a default simulation page. */
        NCSimulationClass
	    *SimPage = new NCSimulationClass("empty.nc",
					     /* Text control flags. */
					     NCSimMF -> TextNotebook,
					     wxID_ANY,
					     wxT(""),
					     0,
					     /* ZBuffer flags. */
					     GlblZBufferSize,
					     &GlblStockBBox,
					     GlblToolType,
					     GlblToolDiameter,
					     GlblUnits);

	NCSimMF -> TextNotebook -> AddPage(SimPage, wxT("empty.nc"), true);
        NCSimMF -> CrntSim = SimPage;
    }

    if (GlblToolGeomFName != NULL && strlen(GlblToolGeomFName) > 0)
	NCSimMF -> CrntSim -> LoadGeometry(GlblToolGeomFName,
					   NCSIM_OBJ_TOOL);
    if (GlblStockGeomFName != NULL && strlen(GlblStockGeomFName) > 0)
	NCSimMF -> CrntSim -> LoadGeometry(GlblStockGeomFName,
					   NCSIM_OBJ_STOCK);
    if (GlblPartGeomFName != NULL && strlen(GlblPartGeomFName) > 0)
	NCSimMF -> CrntSim -> LoadGeometry(GlblPartGeomFName,
					   NCSIM_OBJ_PART);

    for (i = 0; i < NumFiles; i++) {
	char *p;

	if ((p = strrchr((char *) FileNames[i], '.')) != NULL) {
	    if (stricmp(p, ".itd") == 0 ||
		stricmp(p, ".ibd") == 0 ||
		stricmp(p, ".imd") == 0 ||
		stricmp(p, ".igs") == 0 ||
		stricmp(p, ".iges") == 0 ||
		stricmp(p, ".stl") == 0) {
		NCSimMF -> CrntSim -> LoadGeometry(FileNames[i]);
	    }
	    else if (stricmp(p, ".nc") == 0 ||
		     stricmp(p, ".gc") == 0 ||
		     stricmp(p, ".gcode") == 0) {
		NCSimMF -> CreateNewSim(FileNames[i]);
	    }
	    else
	        NCSimPrintf(NCSIM_LOG_ERROR, "Failed to load file \"%s\".\n",
			    FileNames[i]);
	}
	else
	    NCSimPrintf(NCSIM_LOG_ERROR, "Failed to load file \"%s\".\n",
			FileNames[i]);
    }

    NCSimMF -> LoadGUIState();

    NCSimMF -> CrntSim -> ActivateSimPage();
    NCSimMF -> Refresh();

    /* Start the Event loop. */
    return true;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Constructor of the main frame of NCSim                                   M
*                                                                            *
* PARAMETERS:                                                                M
*   Title:   The title of the frame.                                         M
*   Size:    The size of the frame.                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   NoValue                                                                  M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::NCSimMainFrame                                           M
*****************************************************************************/
NCSimMainFrame::NCSimMainFrame(const wxString &Title, const wxSize &Size):
	wxFrame(NULL, wxID_ANY, Title, wxDefaultPosition, Size)
{
    NCSIM_DEBUG_FUNC_ENTRY

    int i;
    wxMenu
	*StateMenu = new wxMenu,		      /* Create a menu bar. */
	*ViewMenu = new wxMenu,
	*HelpMenu = new wxMenu;
    wxAuiPaneInfo PaneInfo;

    SetIcon(wxIcon(NCSimLogo));				  /* Sets the logo. */

    PPanel = new wxPanel(this);
    AuiManager = new wxAuiManager(PPanel,
				  wxAUI_MGR_DEFAULT |
				      wxAUI_MGR_TRANSPARENT_DRAG);

    for (i = 0; i < NCSIM_MAX_NUM_RECENT_LOADED_FILES; i++)
	RecentlyLoadedFiles[i] = NULL;

    FileMenu = new wxMenu;
    UpdateFileMenu();

    ViewMenu -> Append(wxID_NCSIM_VIEW_MENU_VIEW_UP,
		       wxT("&Up View\tF5"),
		       wxT("Reset the view of the simulation to up view"));
    ViewMenu -> Append(wxID_NCSIM_VIEW_MENU_VIEW_FRONT,
		       wxT("&Front View\tF6"),
		       wxT("Reset the view of the simulation to front view"));
    ViewMenu -> Append(wxID_NCSIM_VIEW_MENU_VIEW_SIDE,
		       wxT("&Side View\tF7"),
		       wxT("Reset the view of the simulation to side view"));
    ViewMenu -> Append(wxID_NCSIM_VIEW_MENU_RESCALE_VIEW,
		       wxT("&Autoscale\tF8"),
		       wxT("Autoscale to refit geometry in screen"));

    StateMenu -> Append(wxID_NCSIM_STATE_MENU_SET_ALL_PARAMS,
			wxT("Simulation &Parameters\tAlt-P"),
			wxT("Set all simulation parameters"));
    StateMenu -> Append(wxID_NCSIM_STATE_MENU_SET_OGL_PARAMS,
			wxT("&Rendering Parameters\tAlt-R"),
			wxT("Set all rendering parameters"));

    HelpMenu -> Append(wxID_NCSIM_HELP_MENU_ABOUT, wxT("&About...\tF1"),
		       wxT("Show about dialog"));

    MenuBar = new wxMenuBar();

    MenuBar -> Append(FileMenu, wxT("&File"));
    MenuBar -> Append(ViewMenu, wxT("&View"));
    MenuBar -> Append(StateMenu, wxT("&State"));
    MenuBar -> Append(HelpMenu, wxT("&Help"));

    /* ... and attach this menu bar to the frame. */
    SetMenuBar(MenuBar);

    /* Add a tool bar. */
    TBar = new wxToolBar(this, wxID_ANY);
    TBar -> SetToolBitmapSize(wxSize(24, 24));

    wxBitmap ButtonFwrdStep(NCSimIconFrwdStep),
	     ButtonFwrd(NCSimIconFrwd),
	     ButtonFFwrd(NCSimIconFFrwd),
	     ButtonBwrdStep(NCSimIconBwrdStep),
	     ButtonBwrd(NCSimIconBwrd),
	     ButtonFBwrd(NCSimIconFBwrd),
	     ButtonStop(NCSimIconStop),
	     ButtonPause(NCSimIconPause);

    TBar -> AddTool(wxID_NCSIM_ANIM_FBWRD, wxEmptyString, ButtonFBwrd,
		    wxNullBitmap, wxITEM_NORMAL,
		    NCSimToolTip(wxID_NCSIM_ANIM_FBWRD));
    TBar -> AddTool(wxID_NCSIM_ANIM_BWRD, wxEmptyString, ButtonBwrd,
		    wxNullBitmap, wxITEM_NORMAL,
		    NCSimToolTip(wxID_NCSIM_ANIM_BWRD));
    TBar -> AddTool(wxID_NCSIM_ANIM_BWRD_STEP, wxEmptyString, ButtonBwrdStep,
		    wxNullBitmap, wxITEM_NORMAL,
		    NCSimToolTip(wxID_NCSIM_ANIM_BWRD_STEP));
    TBar -> AddTool(wxID_NCSIM_ANIM_STOP, wxEmptyString, ButtonStop,
		    wxNullBitmap, wxITEM_NORMAL,
		    NCSimToolTip(wxID_NCSIM_ANIM_STOP));
    TBar -> AddTool(wxID_NCSIM_ANIM_PAUSE, wxEmptyString, ButtonPause,
		    wxNullBitmap, wxITEM_NORMAL,
		    NCSimToolTip(wxID_NCSIM_ANIM_PAUSE));
    TBar -> AddTool(wxID_NCSIM_ANIM_FWRD_STEP, wxEmptyString, ButtonFwrdStep,
		    wxNullBitmap, wxITEM_NORMAL,
		    NCSimToolTip(wxID_NCSIM_ANIM_FWRD_STEP));
    TBar -> AddTool(wxID_NCSIM_ANIM_FWRD, wxEmptyString, ButtonFwrd,
		    wxNullBitmap, wxITEM_NORMAL,
		    NCSimToolTip(wxID_NCSIM_ANIM_FWRD));
    TBar -> AddTool(wxID_NCSIM_ANIM_FFWRD, wxEmptyString, ButtonFFwrd,
		    wxNullBitmap, wxITEM_NORMAL,
		    NCSimToolTip(wxID_NCSIM_ANIM_FFWRD));
    TBar -> AddSeparator();

    AnimSlider = new wxSlider(TBar, wxID_NCSIM_ANIM_SLIDER, 0, 0, 10000,
			      wxDefaultPosition, wxSize(300, -1),
			      wxSL_HORIZONTAL | wxSL_AUTOTICKS);
    AnimSlider -> SetToolTip(NCSimToolTip(wxID_NCSIM_ANIM_SLIDER));
    TBar -> AddControl(AnimSlider);

    AnimNCLineNum = new wxSpinCtrl(TBar, wxID_NCSIM_ANIM_LINE_NUM, wxT(""),
			   wxDefaultPosition, wxSize(80, 20),
			   wxTE_RICH | wxSP_ARROW_KEYS | wxSP_WRAP, 0, 0, 0);
    AnimNCLineNum -> SetToolTip(NCSimToolTip(wxID_NCSIM_ANIM_LINE_NUM));
    TBar -> AddControl(AnimNCLineNum);

    AnimNCText = new wxTextCtrl(TBar, wxID_NCSIM_ANIM_NC_TEXT, wxT(""),
				wxDefaultPosition, wxSize(300, -1),
				wxTE_READONLY);
    AnimNCText -> SetToolTip(NCSimToolTip(wxID_NCSIM_ANIM_NC_TEXT));
    TBar -> AddControl(AnimNCText);

    TBar -> Realize();
    SetToolBar(TBar);

    /* Sets drag and drop catcher. */
    SetDropTarget(new NCSimDnDFileClass);

    /* Create a status bar. */
    StatusBar = new wxStatusBar(this, wxID_ANY, wxST_SIZEGRIP);
    int Widths[] = { -1, -200, -200, -120 };
    StatusBar -> SetFieldsCount(WXSIZEOF(Widths), Widths);
    SetStatusBar(StatusBar);

    Show(true);                   /* Open GL must operated with Show true... */

    OGLWin = new NCSimOGLClass(PPanel, wxID_ANY, wxDefaultPosition,
			       wxDefaultSize,
			       WX_GL_RGBA | WX_GL_DOUBLEBUFFER |
				   wxDOUBLE_BORDER | wxWANTS_CHARS);

    TextNotebook = new wxAuiNotebook(PPanel, wxID_ANY, wxDefaultPosition,
				     wxSize(700, 150),
				     wxAUI_NB_TOP | 
					 wxAUI_NB_TAB_SPLIT | 
				         wxAUI_NB_TAB_MOVE | 
				         wxAUI_NB_SCROLL_BUTTONS | 
				         wxAUI_NB_WINDOWLIST_BUTTON |
				         wxAUI_NB_TAB_EXTERNAL_MOVE);

    LogWin = new NCSimMessagesClass(TextNotebook, wxID_ANY);
    TextNotebook -> AddPage(LogWin, wxT("Info"), false);

    /* Manage the windows we have. */
    PaneInfo.Center();
    PaneInfo.CloseButton(false);
    PaneInfo.Floatable(false);
    PaneInfo.Gripper(false);
    PaneInfo.PaneBorder(false);
    PaneInfo.Movable(false);
    PaneInfo.CaptionVisible(false);
    PaneInfo.Caption(wxT("Display"));
    PaneInfo.Name(wxT("Display"));
    AuiManager -> AddPane(OGLWin, PaneInfo);

    PaneInfo.Top();
    PaneInfo.Floatable(true);
    PaneInfo.Gripper(true);
    PaneInfo.PinButton(false);
    PaneInfo.PaneBorder(true);
    PaneInfo.Movable(true);
    PaneInfo.MinimizeButton(false);
    PaneInfo.MaximizeButton(false);
    PaneInfo.Caption(wxT("CNC code and general information"));
    PaneInfo.Name(wxT("Text"));
    AuiManager -> AddPane(TextNotebook, PaneInfo);

    AuiManager -> Update();

    UseOGLDispLists = GlblUseOGLDispLists != FALSE;

    CrntSim = NULL;

    Show(true);
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Destructor of the main frame of NCSim.                                   M
*                                                                            *
* PARAMETERS:                                                                M
*   None					                             M
*                                                                            *
* RETURN VALUE:                                                              M
*   NoValue                                                                  M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::~NCSimMainFrame                                          M
*****************************************************************************/
NCSimMainFrame::~NCSimMainFrame()
{
    NCSIM_DEBUG_FUNC_ENTRY

    int i;

    SaveGUIState();

    for (i = 0; i < NCSIM_MAX_NUM_RECENT_LOADED_FILES; i++) {
        if (RecentlyLoadedFiles[i] != NULL)
	    IritFree(RecentlyLoadedFiles[i]);
    }

    while (PPanel -> GetEventHandler() != PPanel)
	((wxWindow *) PPanel) -> PopEventHandler(false);
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Update recently read file name in the FileMenu's entries.                M
*                                                                            *
* PARAMETERS:                                                                M
*   RcntFileName:  New read file name to insert to FileMenu's entries.       M
*                                                                            *
* RETURN VALUE:                                                              M
*   bool:  True if new to the list and was indeed inserted, false otherwise. M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::AddFName2RecentFNames                                    M
*****************************************************************************/
bool NCSimMainFrame::AddFName2RecentFNames(const char *RcntFileName)
{
    NCSIM_DEBUG_FUNC_ENTRY

    int i;

    for (i = 0;
	 i < NCSIM_MAX_NUM_RECENT_LOADED_FILES &&
	      RecentlyLoadedFiles[i] != NULL;
	 i++) {
        if (strcmp(RcntFileName, RecentlyLoadedFiles[i]) == 0)
	    return FALSE;
    }

    /* File name was not found in the list - shift existing entries... */
    if (RecentlyLoadedFiles[NCSIM_MAX_NUM_RECENT_LOADED_FILES - 1] != NULL)
        IritFree(RecentlyLoadedFiles[NCSIM_MAX_NUM_RECENT_LOADED_FILES - 1]);
    for (i = NCSIM_MAX_NUM_RECENT_LOADED_FILES - 1; i > 0; i--)
        RecentlyLoadedFiles[i] = RecentlyLoadedFiles[i - 1];

    /* ...and insert new entry as first. */
    RecentlyLoadedFiles[0] = IritStrdup((char *) RcntFileName);

    UpdateFileMenu();	       /* Update the FileMenu with the new entries. */

    return true;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Update recently read file name in the FileMenu's entries.                M
*                                                                            *
* PARAMETERS:                                                                M
*   RcntFileName:  name to delete from list - probably failed to load it.    M
*                                                                            *
* RETURN VALUE:                                                              M
*   bool:  True if deleted, false if not found.				     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::DeleteFName2RecentFNames                                 M
*****************************************************************************/
bool NCSimMainFrame::DeleteFName2RecentFNames(const char *RcntFileName)
{
    NCSIM_DEBUG_FUNC_ENTRY

    int i, j;

    for (i = 0;
	 i < NCSIM_MAX_NUM_RECENT_LOADED_FILES &&
	      RecentlyLoadedFiles[i] != NULL;
	 i++) {
        if (strcmp(RcntFileName, RecentlyLoadedFiles[i]) == 0) {
	    /* Found it - delete. */
	    IritFree(RecentlyLoadedFiles[i]);

	    for (j = i + 1; j < NCSIM_MAX_NUM_RECENT_LOADED_FILES; j++)
	        RecentlyLoadedFiles[j - 1] = RecentlyLoadedFiles[j];
	    RecentlyLoadedFiles[NCSIM_MAX_NUM_RECENT_LOADED_FILES - 1] = NULL;

	    UpdateFileMenu(); /* Update the FileMenu with the new entries. */

	    return true;
	}
    }

    return false;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Update the file menu's entries.                                          M
*                                                                            *
* PARAMETERS:                                                                M
*   None					                             M
*                                                                            *
* RETURN VALUE:                                                              M
*   int:  TRUE if successful, FALSE otherwise.                               M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::UpdateFileMenu                                           M
*****************************************************************************/
int NCSimMainFrame::UpdateFileMenu()
{
    NCSIM_DEBUG_FUNC_ENTRY

    int i,
	n = FileMenu -> GetMenuItemCount();

    for (i = n - 1; i >= 0; i--)
        FileMenu -> Destroy(FileMenu -> FindItemByPosition(i));

    FileMenu -> Append(wxID_NCSIM_FILE_MENU_LOAD_NCFILE,
		       wxT("&Load NC File\tAlt-N"),
		       wxT("Load NC file to simulate"));
    FileMenu -> Append(wxID_NCSIM_FILE_MENU_SAVE_NCFILE,
		       wxT("&Save NC File\tAlt-S"),
		       wxT("Save NC file from simulation"));
    FileMenu -> Append(wxID_NCSIM_FILE_MENU_CLOSE_NCFILE,
		       wxT("&Close NC File\tAlt-C"),
		       wxT("Closed current NC file Simulation"));
    FileMenu -> Append(wxID_NCSIM_FILE_MENU_EXPORT_TPATH,
		       wxT("Export &Tool path\tAlt-T"),
		       wxT("Export current NC simulation's tool path as geometry"));
    FileMenu -> Append(wxID_NCSIM_FILE_MENU_SAVE_NCZBUF,
		       wxT("Save NC &ZBuffer\tAlt-Z"),
		       wxT("Save current NC simulation's Z Buffer as geometry"));
    FileMenu -> Append(wxID_NCSIM_FILE_MENU_IMPORT_FINAL_PART,
		       wxT("&Import Final part Geometry\tAlt-F"),
		       wxT("Import and display the geometry of the final part (optional)"));
    FileMenu -> Append(wxID_NCSIM_FILE_MENU_IMPORT_GEOM,
		       wxT("&Import other Geometry\tAlt-I"),
		       wxT("Import other, general, geomerty, for display only (optional)"));
    FileMenu -> Append(wxID_NCSIM_FILE_MENU_EXIT, wxT("E&xit\tAlt-X"),
		       wxT("Quit this program"));

    if (RecentlyLoadedFiles[0] != NULL) {
        FileMenu -> AppendSeparator();
        for (i = 0;
	     i < NCSIM_MAX_NUM_RECENT_LOADED_FILES &&
	         RecentlyLoadedFiles[i] != NULL;
	     i++) {
	    const char *FName;

	    if (GlblFullRcntFName) {
		FileMenu -> Append(wxID_NCSIM_FILE_MENU_RECENT_FNAME + i,
				   NCSimChars2wxString(RecentlyLoadedFiles[i]));
	    }
	    else {
		NCSIM_NAME_FROM_PATH(FName, RecentlyLoadedFiles[i]);

		FileMenu -> Append(wxID_NCSIM_FILE_MENU_RECENT_FNAME + i,
				   NCSimChars2wxString(FName));
	    }
	}
    }

    return TRUE;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Binds the master setup to a dynamically allocated structure that links   M
* the local class's variables.						     M
*                                                                            *
* PARAMETERS:                                                                M
*   MainWindowPosStr:   Main application window position.                    M
*   MainWindowSizeStr:  Main application window size.			     M
*   WindowsStateStr:    Position and sizes of all windows.                   M
*   RcntFileNamesStrs:  List of recently loaded file names.                  M
*                                                                            *
* RETURN VALUE:                                                              M
*   IritConfigStruct *: A Dynamically allocated and binded Setup config.     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::BindSimGUIConfig                                         M
*****************************************************************************/
IritConfigStruct *NCSimMainFrame::BindSimGUIConfig(char **MainWindowPosStr,
						   char **MainWindowSizeStr,
						   char **WindowsStateStr,
						   char ***RcntFileNamesStrs)
{
    NCSIM_DEBUG_FUNC_ENTRY

    int l = 0;
    IritConfigStruct
	*SetUp = (IritConfigStruct *) IritMalloc(sizeof(IritConfigStruct) *
						     NCSIM_GUI_CFG_NUM_SET_UP +
						     IRIT_LINE_LEN_LONG * 4);
    char
        *StrPtrs = (char *) &SetUp[NCSIM_GUI_CFG_NUM_SET_UP]; 

    /* Copy master setup so we can set its slots. */
    IRIT_GEN_COPY(SetUp, NCSimGUICfgSetUp, sizeof(IritConfigStruct) *
						     NCSIM_GUI_CFG_NUM_SET_UP);
    *MainWindowPosStr = StrPtrs;
    **MainWindowPosStr = 0;
    StrPtrs += IRIT_LINE_LEN_LONG;

    *MainWindowSizeStr = StrPtrs;
    **MainWindowSizeStr = 0;
    StrPtrs += IRIT_LINE_LEN_LONG;

    *WindowsStateStr = StrPtrs;
    **WindowsStateStr = 0;
    StrPtrs += IRIT_LINE_LEN_LONG;

    *RcntFileNamesStrs = (char **) StrPtrs;
    **RcntFileNamesStrs = NULL;

    SetUp[l++].VarData = MainWindowPosStr;		/* Win Pos */
    SetUp[l++].VarData = MainWindowSizeStr;		/* Win Size */
    SetUp[l++].VarData = WindowsStateStr;		/* WinState */
    SetUp[l++].VarData = RcntFileNamesStrs;		/* RcntFileNames */
    assert(NCSIM_GUI_CFG_NUM_SET_UP == l);

    return SetUp;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Loads the NCSim GUI status from a configuration file.                    M
*                                                                            *
* PARAMETERS:                                                                M
*   None					                             M
*                                                                            *
* RETURN VALUE:                                                              M
*   int:  TRUE if successful, FALSE otherwise.                               M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::LoadGUIState                                             M
*****************************************************************************/
int NCSimMainFrame::LoadGUIState()
{
    NCSIM_DEBUG_FUNC_ENTRY

    char *MainWindowPosStr, *MainWindowSizeStr,
	 *WindowsStateStr, **RcntFileNamesStrs;
    int i, x, y;
    IritConfigStruct *SetUp;

    SetUp = BindSimGUIConfig(&MainWindowPosStr, &MainWindowSizeStr,
			     &WindowsStateStr, &RcntFileNamesStrs);

    /* Load the current GUI configuration. */
    IritConfig(GlblGuiStatusFName, SetUp, NCSIM_GUI_CFG_NUM_SET_UP);

    /* Sets main window position and size. */
    if (MainWindowPosStr != NULL && 
	sscanf(MainWindowPosStr, "%d %d", &x, &y) == 2)
        SetPosition(wxPoint(x, y));
    if (MainWindowSizeStr != NULL &&
	sscanf(MainWindowSizeStr, "%d %d", &x, &y) == 2)
	SetSize(x, y);
    if (WindowsStateStr != NULL && strlen(WindowsStateStr) > 0)
	NCSimMF -> AuiManager -> 
		       LoadPerspective(NCSimChars2wxString(WindowsStateStr));

    if (RcntFileNamesStrs[0] != NULL) {
        for (i = 0;
	     i < NCSIM_MAX_NUM_RECENT_LOADED_FILES &&
	         RcntFileNamesStrs[i] != NULL;
	     i++)
	    RecentlyLoadedFiles[i] = IritStrdup(RcntFileNamesStrs[i]);

	UpdateFileMenu();
    }

    IritFree(SetUp);

    return TRUE;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Saves the NCSim GUI status to a configuration file.                      M
*                                                                            *
* PARAMETERS:                                                                M
*   None					                             M
*                                                                            *
* RETURN VALUE:                                                              M
*   int:  TRUE if successful, FALSE otherwise.                               M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::SaveGUIState                                             M
*****************************************************************************/
int NCSimMainFrame::SaveGUIState()
{
    NCSIM_DEBUG_FUNC_ENTRY

    char *MainWindowPosStr, *MainWindowSizeStr,
	 *WindowsStateStr, **RcntFileNamesStrs;
    int i, RetVal;
    IritConfigStruct *SetUp;
    wxSize Size;
    wxPoint Pos;
    wxString Str;

    SetUp = BindSimGUIConfig(&MainWindowPosStr, &MainWindowSizeStr,
			     &WindowsStateStr, &RcntFileNamesStrs);

    /* Saves main window position and size. */
    Pos = GetPosition();
    sprintf(MainWindowPosStr, "%d %d", Pos.x, Pos.y);

    Size = GetSize();
    sprintf(MainWindowSizeStr, "%d %d", Size.x, Size.y);

    Str = NCSimMF -> AuiManager -> SavePerspective();
    if (Str.Len() > 0)
        WindowsStateStr = (char *) NCSimwxString2Chars(Str);

    for (i = 0; 
	 i < NCSIM_MAX_NUM_RECENT_LOADED_FILES &&
	     RecentlyLoadedFiles[i] != NULL;
	 i++) {
	RcntFileNamesStrs[i] = IritStrdup(RecentlyLoadedFiles[i]);
    }
    RcntFileNamesStrs[i] = NULL;

    /* Save the current GUI configuration. */
    RetVal = IritConfigSave(GlblGuiStatusFName, SetUp,
			    NCSIM_GUI_CFG_NUM_SET_UP);

    for (i = 0;
	 i < NCSIM_MAX_NUM_RECENT_LOADED_FILES && RcntFileNamesStrs[i] != NULL;
	 i++)
        IritFree(RcntFileNamesStrs[i]);
    IritFree(SetUp);

    return RetVal;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Implementation of the About event.                                       M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::OnAbout                                                  M
*****************************************************************************/
void NCSimMainFrame::OnAbout(wxCommandEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    wxString CompilerNumber(wxT("")),
        Author(wxT("Gershon Elber"));
    wxArrayString
        Authors(1, &Author);
    wxAboutDialogInfo AboutInfo;

#ifdef __WINNT__ 
    #if _MSC_VER >= 1600 /* VC++ 2010 */
	CompilerNumber = wxT(", MS VS2010 (or more)");
    #elif _MSC_VER >= 1500 /* VC++ 2008 */
	CompilerNumber = wxT(", MS VS2008");
    #elif _MSC_VER >= 1400 /* VC++ 2005 */
	CompilerNumber = wxT(", MS VS2005");
    #elif _MSC_VER >= 1300 /* VC++ 2003 */
	CompilerNumber = wxT(", MS VS2003");
    #endif
#endif /* _WINNT_ */

    AboutInfo.SetDescription(wxT(NCSIM_PROGRAM_NUMBER) + CompilerNumber);
    AboutInfo.SetCopyright(NCSimChars2wxString(NCSIM_VERSION));
    AboutInfo.SetWebSite(wxT(NCSIM_WEBSITE));
    AboutInfo.SetDevelopers(Authors);

    wxAboutBox(AboutInfo);
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Implementation of the recent file name selection event.                  M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::OnRecentFNames                                           M
*****************************************************************************/
void NCSimMainFrame::OnRecentFNames(wxCommandEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    int Idx = ((int) Event.GetId()) - wxID_NCSIM_FILE_MENU_RECENT_FNAME;

    if (Idx >= 0 &&
	Idx < NCSIM_MAX_NUM_RECENT_LOADED_FILES &&
	RecentlyLoadedFiles[Idx] != NULL) {
	SetCursor(wxCURSOR_WAIT);

	CreateNewSim(RecentlyLoadedFiles[Idx]);

	SetCursor(wxCURSOR_ARROW);

	NCSimMF -> CrntSim -> ActivateSimPage();
	NCSimMF -> Refresh();
    }
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   A simple pop up messaging scheme.                                        M
*                                                                            *
* PARAMETERS:                                                                M
*   Msg:    The message itself.						     M
*   Style:  Info, warning, or error.                                         M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::PopupMsg                                                 M
*****************************************************************************/
void NCSimMainFrame::PopupMsg(const char *Msg, NCSimLogStyleType Style)
{
    NCSIM_DEBUG_FUNC_ENTRY

    switch (Style) {
	case NCSIM_LOG_INFO:
	    wxMessageBox(NCSimChars2wxString(Msg), wxT("Information"),
			 wxOK | wxICON_INFORMATION, this);
	    break;
	case NCSIM_LOG_WARN:
	    wxMessageBox(NCSimChars2wxString(Msg), wxT("Warning"),
			 wxOK | wxICON_EXCLAMATION, this);
	    break;
	case NCSIM_LOG_ERROR:
	default:
	    wxMessageBox(NCSimChars2wxString(Msg), wxT("Error"),
			 wxOK | wxICON_ERROR, this);
	    break;
    }
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   A simple pop up yes/no scheme.                                           M
*                                                                            *
* PARAMETERS:                                                                M
*   Msg:    The message itself.						     M
*   Style:  Info, warning, or error.                                         M
*                                                                            *
* RETURN VALUE:                                                              M
*   int:  TRUE for yes, FALSE for now.                                       M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::PopupYesNo                                               M
*****************************************************************************/
int NCSimMainFrame::PopupYesNo(const char *Msg, NCSimLogStyleType Style)
{
    NCSIM_DEBUG_FUNC_ENTRY

    int RetVal;
    wxString 
	Str = NCSimChars2wxString(Msg);

    switch (Style) {
	case NCSIM_LOG_INFO:
	    RetVal = wxMessageBox(Str, wxT("Information"), wxYES_NO, this);
	    break;
	case NCSIM_LOG_WARN:
	    RetVal = wxMessageBox(Str, wxT("Warning"), wxYES_NO, this);
	    break;
	case NCSIM_LOG_ERROR:
	default:
	    RetVal = wxMessageBox(Str, wxT("Error"), wxYES_NO, this);
	    break;
    }

    return RetVal == wxYES;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Implementation of the quit event - close everything and quit.            M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::OnQuit                                                   M
*****************************************************************************/
void NCSimMainFrame::OnQuit(wxCommandEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    Close();
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Trap notbook changes event.					             M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::NCSimPageChanged                                         M
*****************************************************************************/
void NCSimMainFrame::NCSimPageChanged(wxAuiNotebookEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    int Idx;
    wxEventType
	EventType = Event.GetEventType();

    if (EventType == wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGING) {
	if ((Idx = Event.GetSelection()) == 0) {       /* The "Info" widget. */
	}
	else {
	    wxAuiNotebook
	        *NB = (wxAuiNotebook *) Event.GetEventObject();
	    NCSimulationClass
	        *SimPage = (NCSimulationClass *) NB -> GetPage(Idx);
	    wxString Title;

	    Title.Printf(wxT("NCSim - %s"), NB -> GetPageText(Idx));
	    NCSimMF -> SetTitle(Title);

	    CrntSim = SimPage;
	    SimPage -> ActivateSimPage();
	    if (SimPage -> GUITextPage < 0)
	        SimPage -> GUITextPage = Idx;
	    OGLWin -> NCSimPageChange();
	    NB -> Refresh();
	}
    }
    else if (EventType == wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED) {
    }
}

/*****************************************************************************
* DESCRIPTION:                                                               M
* Default logging routine of NCSim, printf style.			     M
*                                                                            *
* PARAMETERS:                                                                M
*  Style:      Print stule, info, warning, or error style.                   M
*  va_alist:   Do "man stdarg".                                              M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::Printf, warning trap, error trap                         M
*****************************************************************************/
#ifdef USE_VARARGS
void NCSimMainFrame::Printf(NCSimLogStyleType Style, char *va_alist, ...)
{
    NCSIM_DEBUG_FUNC_ENTRY

    char *Format, Line[LINE_LEN_VLONG];
    long l;
    va_list ArgPtr;

    va_start(ArgPtr);
    Format = va_arg(ArgPtr, char *);
#else
void NCSimMainFrame::Printf(NCSimLogStyleType Style, char *Format, ...)
{
    NCSIM_DEBUG_FUNC_ENTRY

    char Line[IRIT_LINE_LEN_VLONG];
    long l;
    va_list ArgPtr;

    va_start(ArgPtr, Format);
#endif /* USE_VARARGS */

    vsprintf(Line, Format, ArgPtr);
    va_end(ArgPtr);

    if (NCSimMF == NULL) {
	/* Not initialized yet - print to stderr instead. */
	fprintf(stderr, Line);
	return;
    }

    switch (Style) {
	case NCSIM_LOG_INFO:
	    NCSimMF -> LogWin ->
	        SetDefaultStyle(wxTextAttr(NCSIM_COLOR_INFO));
	    break;
	case NCSIM_LOG_WARN:
	    NCSimMF -> LogWin ->
	        SetDefaultStyle(wxTextAttr(NCSIM_COLOR_WARN));
	    break;
	case NCSIM_LOG_ERROR:
	    NCSimMF -> LogWin ->
	        SetDefaultStyle(wxTextAttr(NCSIM_COLOR_ERROR));
	    break;
    }

    NCSimMF -> LogWin -> AppendText(NCSimChars2wxString(Line));

    /* Make the messages page active and the line visible. */
    l = NCSimMF -> LogWin -> GetInsertionPoint();
    NCSimMF -> LogWin -> ShowPosition(l > 0 ? l - 1 : l);
}

/*****************************************************************************
* DESCRIPTION:                                                               *
*   Call back function of the NC GCode parser, in case of parsing problems.  *
*                                                                            *
* PARAMETERS:                                                                *
*   Line:   Reported problem.                                                *
*                                                                            *
* RETURN VALUE:                                                              *
*   void                                                                     *
*****************************************************************************/
static void UserNCGCodeParserError(char *Line)
{
    NCSIM_DEBUG_FUNC_ENTRY

    NCSimMF -> Printf(NCSIM_LOG_WARN, "NC parser: %s\n", Line);
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Create a new simulation, given an NC file.				     M
*                                                                            *
* PARAMETERS:                                                                M
*   FullPath: Full path to file to read.                                     *
*                                                                            *
* RETURN VALUE:                                                              M
*   bool:   True if successful, false otherwise.                             M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::CreateNewSim                                             M
*****************************************************************************/
bool NCSimMainFrame::CreateNewSim(const char *FullPath)
{
    NCSIM_DEBUG_FUNC_ENTRY

    int TextBufIdx = 0;
    FILE *f;
    char Line[IRIT_LINE_LEN_VLONG];
    const char *FName;
    wxWindowUpdateLocker
	NoUpdates(TextNotebook);

    NCSIM_NAME_FROM_PATH(FName, FullPath);

    SetCursor(wxCURSOR_WAIT);

    NCSimMF -> Printf(NCSIM_LOG_INFO, "NC parser: reading file \"%s\":\n",
		      FullPath);
    if ((f = fopen(FullPath, "r")) != NULL) {
        int i,
	    LineNum = 1;
	NCSimulationClass      /* Initialize the simulation's current page. */
	    *SimPage = new NCSimulationClass(FullPath,
					     /* Text control flags. */
					     TextNotebook,
					     wxID_ANY,
					     wxT(""),
					     GlblRichTextCtrlNCFile ?
					         (wxTE_RICH | wxTE_RICH2) : 0,
					     /* ZBuffer flags. */
					     GlblZBufferSize,
					     &GlblStockBBox,
					     GlblToolType,
					     GlblToolDiameter,
					     GlblUnits);
	wxString TextBuffer, LineBuffer;

	TextNotebook -> AddPage(SimPage, NCSimChars2wxString(FName),
				false);			/* Add to notebook. */
	TextNotebook -> Freeze();

	SimPage -> SetNCFileParams();
	SimPage -> SetDefaultStyle(wxTextAttr(NCSIM_COLOR_NORMAL));
	SimPage -> GStream = IPNCGCodeParserInit(SimPage -> HasRelArcCenter,
	    SimPage -> Units == NCSIM_UNIT_INCHES ? SimPage -> FeedRate / 60.0
						  : SimPage -> FeedRate,
	    SimPage -> SpindleSpeed,
	    SimPage -> ToolIdx,
	    SimPage -> ReverseZDir,
	    UserNCGCodeParserError);

	AnimNCLineNum -> SetRange(1, 100000000);
	while (fgets(Line, IRIT_LINE_LEN_VLONG, f) != NULL) {
	    /* Accumulate text buffer and update infrequently (expensive). */
	    LineBuffer.Printf(wxT("%4d | %s"), LineNum, Line);
	    TextBuffer.Append(LineBuffer);

	    if (LineNum % 1000 == 0) {
	        AnimNCLineNum -> SetValue(LineNum);
		Update();
		Refresh();
	    }
	    IPNCGCodeParserParseLine(SimPage -> GStream, Line, LineNum++);
	}
	SimPage -> AppendText(TextBuffer);

	AnimNCLineNum -> SetValue(LineNum);

	if (!feof(f) || !IPNCGCodeParserDone(SimPage -> GStream)) {
	    Printf(NCSIM_LOG_ERROR,
		   "Invalid file - premature termination of file \"%s\" during reading!?\n",
		   FullPath);
	    delete SimPage;

	    SetCursor(wxCURSOR_ARROW);
	    return false;
	}
	else {
	    CrntSim = SimPage;
	}

	fclose(f);
	NCSimMF -> Printf(NCSIM_LOG_INFO,
			  "NC parser: reading file \"%s\" complete.\n", FullPath);

	/* Update the geometry of this page - tool path. */
	SimPage -> GPathGeom = IPNCGCode2Geometry(SimPage -> GStream);

	if (IRIT_PT_EQ_ZERO(SimPage -> StockBBox.Min) &&
	    IRIT_PT_EQ_ZERO(SimPage -> StockBBox.Max))
	    SimPage -> SetBBoxFromPathGeom(
		 SimPage -> Tools[SimPage -> ToolIdx].Diameter / 2.0, 0.1);

	SimPage -> ReCreateStockGeometry();
	SimPage -> UpdateLinePosMap();
	SimPage -> ActivateSimPage(true);
	SimPage -> ShowPosition(0);

	/* If we are successful and we have an empty.nc page - delete it. */
	for (i = TextNotebook -> GetPageCount() - 1; i > 0; i--) {
	    wxString
		Str = TextNotebook -> GetPageText(i);

	    if (strcmp("empty.nc", NCSimwxString2Chars(Str)) == 0)
		TextNotebook -> DeletePage(i);
	}

	/* Update the recently loaded files in the FileMenu. */
	AddFName2RecentFNames(FullPath);

	/* wxWidgets 2.91 has a bug refreshing the wxTextCtrl if made the   */
	/* top (selected) one.  Hence we are forced to expose the Info page */
        /* reporting on the loading process which is not a bad alternative. */
	TextNotebook -> Thaw();
	TextNotebook -> SetSelection(0);
	TextNotebook -> Refresh();

	SetCursor(wxCURSOR_ARROW);

	return true;
    }
    else {
        sprintf(Line, "Failed to read file \"%s\"", FullPath);
	PopupMsg(Line, NCSIM_LOG_WARN);

        Printf(NCSIM_LOG_ERROR, "Failed to read file \"%s\"\n", FullPath);

	DeleteFName2RecentFNames(FullPath);

	SetCursor(wxCURSOR_ARROW);
	return false;
    }
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Import general geometry.						     M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void		                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::ImportGeometry					     M
*****************************************************************************/
void NCSimMainFrame::ImportGeometry(wxCommandEvent &Event)
{
    const char
	*HelpStr = "Irit text files (*.itd)|*.itd|"
		   "Irit binary files (*.ibd)|*.ibd|"
		   "IGES files (*.igs)|*.igs|"
		   "IGES files (*.iges)|*.iges|"
		   "STL files (*.stl)|*.stl|"
		   "All files (*.*)|*.*";

    NCSIM_DEBUG_FUNC_ENTRY

    wxFileDialog 
	*FileDlg = new wxFileDialog(this,
	    wxT("Please choose a geometry file"), 
	    wxT(""),        /* Default directory */
	    wxT(""),	       /* Default file */
	    NCSimChars2wxString(HelpStr),
	    wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_CHANGE_DIR);
    
    if (wxID_OK == FileDlg -> ShowModal()) {
	wxString
	    Str = FileDlg -> GetPath();

	SetCursor(wxCURSOR_WAIT);

	NCSimMF -> CrntSim -> LoadGeometry(NCSimwxString2Chars(Str));

	SetCursor(wxCURSOR_ARROW);

	NCSimMF -> CrntSim -> ActivateSimPage();
	NCSimMF -> Refresh();
    }

    delete FileDlg;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Import the geometry of the final part.				     M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void		                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::ImportFinalPart					     M
*****************************************************************************/
void NCSimMainFrame::ImportFinalPart(wxCommandEvent &Event)
{       
    NCSIM_DEBUG_FUNC_ENTRY

    const char
	*HelpStr = "Irit text files (*.itd)|*.itd|"
		   "Irit binary files (*.ibd)|*.ibd|"
		   "IGES files (*.igs)|*.igs|"
		   "IGES files (*.iges)|*.iges|"
		   "STL files (*.stl)|*.stl|"
		   "All files (*.*)|*.*";
    wxFileDialog 
	*FileDlg = new wxFileDialog(this,
	    wxT("Please choose a final part geometry file"), 
	    wxT(""),           /* Default directory */
	    wxT(""),	       /* Default file */
	    NCSimChars2wxString(HelpStr),
	    wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_CHANGE_DIR);
    
    if (wxID_OK == FileDlg -> ShowModal()) {
	wxString
	    Str = FileDlg -> GetPath();

	SetCursor(wxCURSOR_WAIT);

	NCSimMF -> CrntSim ->
		      LoadGeometry(NCSimwxString2Chars(Str), NCSIM_OBJ_PART);

	SetCursor(wxCURSOR_ARROW);

	NCSimMF -> CrntSim -> ActivateSimPage();
	NCSimMF -> Refresh();
    }

    delete FileDlg;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Load NC File to simulate.						     M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void		                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::LoadNCFile  					     M
*****************************************************************************/
void NCSimMainFrame::LoadNCFile(wxCommandEvent &Event)
{       
    NCSIM_DEBUG_FUNC_ENTRY

    const char
	*HelpStr = "NC files (*.nc)|*.nc|"
		   "NC files (*.ncc)|*.ncc|"
		   "NC files (*.cnc)|*.cnc|"
		   "NC files (*.gcode)|*.gcode|"
		   "NC files (*.gc)|*.gc|"
		   "All files (*.*)|*.*";
    wxFileDialog 
	*FileDlg = new wxFileDialog(this,
	    wxT("Please choose an NC file"), 
	    wxT(""),        /* Default directory */
	    wxT(""),	         /* Default file */
	    NCSimChars2wxString(HelpStr),
	    wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_CHANGE_DIR);
    
    if (wxID_OK == FileDlg -> ShowModal()) {
	wxString
	    Str = FileDlg -> GetPath();

	CreateNewSim(NCSimwxString2Chars(Str));

	NCSimMF -> CrntSim -> ActivateSimPage();
	NCSimMF -> Refresh();
    }

    delete FileDlg;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Save NC File from simulation.					     M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void		                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::SaveNCFile  					     M
*****************************************************************************/
void NCSimMainFrame::SaveNCFile(wxCommandEvent &Event)
{       
    NCSIM_DEBUG_FUNC_ENTRY

    const char
	*HelpStr = "NC files (*.nc)|*.nc|"
		   "NC files (*.ncc)|*.ncc|"
		   "NC files (*.cnc)|*.cnc|"
		   "NC files (*.gcode)|*.gcode|"
		   "NC files (*.gc)|*.gc|"
		   "All files (*.*)|*.*";
    wxFileDialog 
	*FileDlg = new wxFileDialog(this,
	    wxT("Please choose an NC file to save to"), 
	    wxT(""),        /* Default directory */
	    wxT(""),	       /* Default file */
	    NCSimChars2wxString(HelpStr),
	    wxFD_SAVE | wxFD_OVERWRITE_PROMPT | wxFD_CHANGE_DIR);
    
    if (wxID_OK == FileDlg -> ShowModal()) {
	wxString
	    Str = FileDlg -> GetPath();

	SetCursor(wxCURSOR_WAIT);

	IPNCGCodeSave2File(CrntSim -> GStream,
			   NCSimwxString2Chars(Str));

	SetCursor(wxCURSOR_ARROW);
    }

    delete FileDlg;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Close current NC File.						     M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void		                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::CloseNCFile  					     M
*****************************************************************************/
void NCSimMainFrame::CloseNCFile(wxCommandEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    int PageNum = TextNotebook -> GetSelection();
    char Line[IRIT_LINE_LEN_LONG];
    wxString
	PageTitle = TextNotebook -> GetPageText(PageNum);
    NCSimulationClass *SimPage;

    if (PageNum == 0) {
        Printf(NCSIM_LOG_ERROR, "Cannot delete \"Info\" page\n");
	return;
    }

    if (TextNotebook -> GetPageCount() <= 2) {
        Printf(NCSIM_LOG_ERROR, "Cannot delete last NC simulation file\n");
	return;
    }

    sprintf(Line, "Are you sure you want to delete NC file \"%s\"",
	    NCSimwxString2Chars(PageTitle));
    if (!NCSimMF -> PopupYesNo(Line))
	return;

    NCSimMF -> TextNotebook -> DeletePage(PageNum);

    /* Select a new NC file. */
    SimPage =  (NCSimulationClass *) TextNotebook -> GetPage(1);
    sprintf(Line, "NCSim - %s", TextNotebook -> GetPageText(1));
    NCSimMF -> SetTitle(NCSimChars2wxString(Line));

    CrntSim = SimPage;
    SimPage -> ActivateSimPage();

    NCSimMF -> Refresh();
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Saves the current geometry of the tool path as an ITD or IGS geometry    M
* file.									     M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::SaveNCTPathToFile                                        M
*****************************************************************************/
void NCSimMainFrame::SaveNCTPathToFile(wxCommandEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    const char *FName,
	*HelpStr = "Irit Text Data files (*.itd)|*.itd|"
		   "IGES files (*.igs)|*.igs|"
		   "All files (*.*)|*.*";
    FILE *f;
    wxFileDialog *FileDlg;

    if (CrntSim -> GPathGeom == NULL) {
        NCSimPrintf(NCSIM_LOG_ERROR, "No tool path to save to a file.\n");
	return;
    }

    FileDlg = new wxFileDialog(this,
	    wxT("Please choose a file name to save tool path"), 
	    wxT(""),        /* Default directory */
	    wxT(""),	       /* Default file */
	    NCSimChars2wxString(HelpStr),
	    wxFD_SAVE | wxFD_OVERWRITE_PROMPT | wxFD_CHANGE_DIR);

    if (wxID_OK == FileDlg -> ShowModal()) {
	wxString
	    Str = FileDlg -> GetPath();

	if ((f = fopen(FName = NCSimwxString2Chars(Str), "w")) != NULL) {
	    if (strlen(FName) > 4 &&
	        stricmp(&FName[strlen(FName) - 4], ".igs") == 0) {
	        IrtHmgnMatType Mat;

	        fclose(f);
	        MatGenUnitMat(Mat);
	        IPIgesSaveFile(CrntSim -> GPathGeom, Mat,
			      (char *) FName, FALSE);
	    }
	    else { /* Save as itd file. */
	        IPPutObjectToFile(f, CrntSim -> GPathGeom, FALSE);
	        fclose(f);
	    }
	}
    }

    delete FileDlg;
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Saves the current geometry of the NC ZBuffer as an ITD or STL geometry   M
* file.									     M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::SaveNCZBufToFile                                         M
*****************************************************************************/
void NCSimMainFrame::SaveNCZBufToFile(wxCommandEvent &Event)
{
    const char
	*HelpStr = "Irit Text Data files (*.itd)|*.itd|"
		   "Stereo Lithography files (*.stl)|*.stl|"
		   "All files (*.*)|*.*";
    NCSIM_DEBUG_FUNC_ENTRY

    wxFileDialog
        *FileDlg = new wxFileDialog(this,
	    wxT("Please choose a file name to save Z buffer"), 
	    wxT(""),           /* Default directory */
	    wxT(""),	       /* Default file */
	    NCSimChars2wxString(HelpStr),
	    wxFD_SAVE | wxFD_OVERWRITE_PROMPT | wxFD_CHANGE_DIR);

    if (wxID_OK == FileDlg -> ShowModal()) {
	wxString
	    Str = FileDlg -> GetPath();

	if ((GlblNCZBufSaveFile = fopen(NCSimwxString2Chars(Str), "w")) != NULL) {
	    const char
		*FName = NCSimwxString2Chars(FileDlg -> GetPath());
	    int FNameLen = strlen(FName);

	    SetCursor(wxCURSOR_WAIT);

	    if (FNameLen > 4 && stricmp(&FName[FNameLen - 4], ".itd") == 0) {
		GlblNCZBufSaveITD = TRUE;
		fprintf(GlblNCZBufSaveFile, "[OBJECT NCZBUF\n");
	    }
	    else { /* Save STL */
		GlblNCZBufSaveITD = FALSE;
		fprintf(GlblNCZBufSaveFile, "solid nczbuf\n");
	    }

	    CrntSim -> ZBufferGetCellTris(0, 0, SaveOneNCZBufferTriangle);

	    if (GlblNCZBufSaveITD) {
		fprintf(GlblNCZBufSaveFile, "]\n");
	    }
	    else { /* Save STL */
	    	fprintf(GlblNCZBufSaveFile, "endsolid\n");
	    }

	    fclose(GlblNCZBufSaveFile);

	    SetCursor(wxCURSOR_ARROW);
	}
    }

    delete FileDlg;
}

/*****************************************************************************
* DESCRIPTION:                                                               *
*   Renders one triangle from the NC ZBuffer, given its three vertices'	     *
* coefficients.  This function is a call back function from the NC ZBuffer.  *
*                                                                            *
* PARAMETERS:                                                                *
*   X1, Y1, Z1:   Coefficient of first triangle vertex.                      *
*   X2, Y2, Z2:   Coefficient of second triangle vertex.                     *
*   X3, Y3, Z3:   Coefficient of third triangle vertex.                      *
*                                                                            *
* RETURN VALUE:                                                              *
*   void                                                                     *
*****************************************************************************/
static void SaveOneNCZBufferTriangle(IrtRType X1,
				     IrtRType Y1,
				     IrtRType Z1,
				     IrtRType X2,
				     IrtRType Y2,
				     IrtRType Z2,
				     IrtRType X3,
				     IrtRType Y3,
				     IrtRType Z3)
{
    NCSIM_DEBUG_FUNC_ENTRY

    if (GlblNCZBufSaveITD) {
	fprintf(GlblNCZBufSaveFile, "    [POLYGON 3\n");
	fprintf(GlblNCZBufSaveFile, "\t[%f %f %f]\n", X1, Y1, Z1);
	fprintf(GlblNCZBufSaveFile, "\t[%f %f %f]\n", X2, Y2, Z2);
	fprintf(GlblNCZBufSaveFile, "\t[%f %f %f]\n", X3, Y3, Z3);
	fprintf(GlblNCZBufSaveFile, "    ]\n");
    }
    else { /* Save STL */
	IrtVecType V1, V2, V;

	V1[0] = X2 - X1;
	V1[1] = Y2 - Z1;
	V1[2] = Z2 - Z1;
	V2[0] = X3 - X1;
	V2[1] = Y3 - Z1;
	V2[2] = Z3 - Z1;
	IRIT_CROSS_PROD(V, V1, V2);
	IRIT_VEC_SAFE_NORMALIZE(V);
	if (V[2] > 0)
	    fprintf(GlblNCZBufSaveFile,
		    "  facet normal %f %f %f\n    outer loop\n",
		    V[0], V[1], V[2]);
	else
	    fprintf(GlblNCZBufSaveFile,
		    "  facet normal %f %f %f\n    outer loop\n",
		    -V[0], -V[1], -V[2]);
	fprintf(GlblNCZBufSaveFile, "      vertex %f %f %f\n", X1, Y1, Z1);
	fprintf(GlblNCZBufSaveFile, "      vertex %f %f %f\n", X2, Y2, Z2);
	fprintf(GlblNCZBufSaveFile, "      vertex %f %f %f\n", X3, Y3, Z3);
	fprintf(GlblNCZBufSaveFile, "    endloop\n  endfacet\n");
    }
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Sets the NC simulation mode - real time based on fixed step based.	     M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* SEE ALSO:                                                                  M
*                                                                            M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::StateViewSimMode                                         M
*****************************************************************************/
void NCSimMainFrame::StateViewSimMode(wxCommandEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    if (CrntSim == NULL || OGLWin == NULL)
	return;

    switch (Event.GetId()) {
	default:
	    assert(0);				   /* Should never get here. */
	    break;
	case wxID_NCSIM_STATE_MENU_SET_ALL_PARAMS:
	    CrntSim -> SetSimInfoParams();
	    break;
	case wxID_NCSIM_STATE_MENU_SET_OGL_PARAMS:
	    OGLWin -> SetOGLParams();
	    break;
	case wxID_NCSIM_VIEW_MENU_VIEW_UP:
	    CrntSim -> AutoScaleView(NCSIM_VIEW_UP);
	    OGLWin -> Refresh();
	    break;
	case wxID_NCSIM_VIEW_MENU_VIEW_FRONT:
	    CrntSim -> AutoScaleView(NCSIM_VIEW_FRONT);
	    OGLWin -> Refresh();
	    break;
	case wxID_NCSIM_VIEW_MENU_VIEW_SIDE:
	    CrntSim -> AutoScaleView(NCSIM_VIEW_SIDE);
	    OGLWin -> Refresh();
	    break;
	case wxID_NCSIM_VIEW_MENU_RESCALE_VIEW:
	    CrntSim -> AutoScaleView(NCSIM_VIEW_CRNT);
	    OGLWin -> Refresh();
	    break;
    }
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Captures NC simulation animation controls.                               M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* SEE ALSO:                                                                  M
*                                                                            M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::AnimControl                                              M
*****************************************************************************/
void NCSimMainFrame::AnimControl(wxCommandEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    CrntSim -> SingleStepActive = FALSE;

    switch (Event.GetId()) {
	case wxID_NCSIM_ANIM_FBWRD:
	    CrntSim -> AnimMultFactor = CrntSim -> AnimMultFactor > 0 ?
		CrntSim -> AnimMultFactor * 0.5 :
		CrntSim -> AnimMultFactor * 2.0;
	    CrntSim -> AnimState(NCSIM_ANIM_STATE_RUN);
	    break;
        case wxID_NCSIM_ANIM_BWRD:
	    CrntSim -> AnimMultFactor = -1.0;
	    CrntSim -> AnimState(NCSIM_ANIM_STATE_RUN);
	    break;
	case wxID_NCSIM_ANIM_BWRD_STEP:
	    CrntSim -> SingleStepActive = TRUE;
	    CrntSim -> AnimMultFactor = -1.0;
	    CrntSim -> AnimState(NCSIM_ANIM_STATE_RUN);
	    break;
        default:
	    assert(0);				  /* Should never get here. */
        case wxID_NCSIM_ANIM_STOP:
	    CrntSim -> AnimState(NCSIM_ANIM_STATE_STOP);
	    break;
        case wxID_NCSIM_ANIM_PAUSE:
	    CrntSim -> AnimState(NCSIM_ANIM_STATE_PAUSE);
	    break;
	case wxID_NCSIM_ANIM_FWRD_STEP:
	    CrntSim -> SingleStepActive = TRUE;
	    CrntSim -> AnimMultFactor = 1.0;
	    CrntSim -> AnimState(NCSIM_ANIM_STATE_RUN);
	    break;
        case wxID_NCSIM_ANIM_FWRD:
	    CrntSim -> AnimMultFactor = 1.0;
	    CrntSim -> AnimState(NCSIM_ANIM_STATE_RUN);
	    break;
        case wxID_NCSIM_ANIM_FFWRD:
	    CrntSim -> AnimMultFactor = CrntSim -> AnimMultFactor < 0 ?
		CrntSim -> AnimMultFactor * 0.5 :
		CrntSim -> AnimMultFactor * 2.0;
	    CrntSim -> AnimState(NCSIM_ANIM_STATE_RUN);
	    break;
    }
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Show help line in the status bar for the menu items.                     M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* SEE ALSO:                                                                  M
*                                                                            M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::ShowMenuHelp                                             M
*****************************************************************************/
void NCSimMainFrame::ShowMenuHelp(wxMenuEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    wxMenuBar
        *MenuBar = GetMenuBar();
    int Id = Event.GetMenuId();

    if (Id >= wxID_NCSIM_MENU_START && Id <= wxID_NCSIM_MENU_END)
	StatusBar -> SetStatusText(MenuBar -> GetHelpString(Id), 2);
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Captures NC simulation animation controls.                               M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* SEE ALSO:                                                                  M
*                                                                            M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::AnimSliderControl                                        M
*****************************************************************************/
void NCSimMainFrame::AnimSliderControl(wxCommandEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    int Val = AnimSlider -> GetValue();

    CrntSim -> SetAnimPercent(Val / 100.0);
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Traps events from text controllers.                                      M
*                                                                            *
* PARAMETERS:                                                                M
*   Event: N.S.F.I.                                                          M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::OnTextChange	                                     M
*****************************************************************************/
void NCSimMainFrame::OnTextChange(wxCommandEvent &Event)
{
    NCSIM_DEBUG_FUNC_ENTRY

    if (Event.GetId() == wxID_NCSIM_ANIM_LINE_NUM) {
        int i;
	wxString
	    Str = Event.GetString();
	const char
	    *s = NCSimwxString2Chars(Str);

	if (sscanf(s, "%d", &i) == 1) {
	    CrntSim -> SetAnimLine(i);
	}
    }
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Update the frames per second status report of NCSim.                     M
*                                                                            *
* PARAMETERS:                                                                M
*   FPS:       Frame per second performace.                                  M
*   NumPolys:  Number of rendered polys in scene.                            M
*                                                                            *
* RETURN VALUE:                                                              M
*   void                                                                     M
*                                                                            *
* SEE ALSO:                                                                  M
*                                                                            M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimMainFrame::SetFPSNumPolysStatus                                     M
*****************************************************************************/
void NCSimMainFrame::SetFPSNumPolysStatus(IrtRType FPS, int NumPolys)
{
    NCSIM_DEBUG_FUNC_ENTRY

    char Line[IRIT_LINE_LEN];

    sprintf(Line, "%.2f FPS, %d Polys", FPS, NumPolys);

    StatusBar -> SetStatusText(NCSimChars2wxString(Line), 3);
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Constructor for the main frame split control.                            M
*                                                                            *
* PARAMETERS:                                                                M
*   Parent:  Window, the main frame itself.                                  M
*                                                                            *
* RETURN VALUE:                                                              M
*   NoValue                                                                  M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimSplitMainFrame::NCSimSplitMainFrame                                 M
*****************************************************************************/
NCSimSplitMainFrame::NCSimSplitMainFrame(wxFrame *Parent):
	wxSplitterWindow(Parent,
			 wxID_ANY,
			 wxDefaultPosition,
			 wxDefaultSize,
                         wxSP_3D | wxSP_LIVE_UPDATE | wxCLIP_CHILDREN)
{
    NCSIM_DEBUG_FUNC_ENTRY
}

/*****************************************************************************
* DESCRIPTION:                                                               M
*   Handles file names drag and drop.                                        M
*                                                                            *
* PARAMETERS:                                                                M
*   x, y:      Dropping location.  Currently not used.                       M
*   Filenames: List of dropped file names.                                   M
*                                                                            *
* RETURN VALUE:                                                              M
*   bool:     TRUE if succesfully loaded the files, FALSE otherwise.         M
*                                                                            *
* SEE ALSO:                                                                  M
*                                                                            M
*                                                                            *
* KEYWORDS:                                                                  M
*   NCSimDnDFileClass::OnDropFiles                                           M
*****************************************************************************/
bool NCSimDnDFileClass::OnDropFiles(wxCoord x, 
				    wxCoord y, 
				    const wxArrayString &Filenames)
{
    NCSIM_DEBUG_FUNC_ENTRY

    const char *FilePath, *p;
    int i,
	NumFiles = Filenames.GetCount();     

    for (i = 0; i < NumFiles; i++) {
	FilePath = NCSimwxString2Chars(Filenames[i]);	
	if ((p = strrchr((char *) FilePath, '.')) != NULL) {
	    if (stricmp(p, ".itd") == 0 ||
		stricmp(p, ".ibd") == 0 ||
		stricmp(p, ".imd") == 0 ||
		stricmp(p, ".igs") == 0 ||
		stricmp(p, ".iges") == 0 ||
		stricmp(p, ".stl") == 0) {
		if (!NCSimMF -> CrntSim -> LoadGeometry(FilePath))
		    return false;
	    }
	    else if (stricmp(p, ".nc") == 0 ||
		     stricmp(p, ".gc") == 0 ||
		     stricmp(p, ".gcode") == 0) {
		if (!NCSimMF -> CreateNewSim(FilePath))
		    return false;
	    }
	    else {
	        NCSimPrintf(NCSIM_LOG_ERROR, "Failed to load file \"%s\".\n",
			    FilePath);
		return false;
	    }
	}
	else {
	    NCSimPrintf(NCSIM_LOG_ERROR, "Failed to load file \"%s\".\n",
			FilePath);
	    return false;
	}
    }

    NCSimMF -> CrntSim -> ActivateSimPage();
    NCSimMF -> Refresh();

    return true;
}

/*****************************************************************************
* AUXILIARY:								     *
*    The functions below are defined through Irit's grap_lib.		     *
*****************************************************************************/
extern "C" {
    void IGIritBeep(void) {}
    void IGPlotTo2D(IrtRType X, IrtRType Y) {}
    void IGMoveTo2D(IrtRType X, IrtRType Y) {}
    void IGLineTo2D(IrtRType X, IrtRType Y) {}
    void IGSetColorIntensity(int High) {}
    int IGCGDrawDTexture(IPObjectStruct *PObj) { return FALSE; }
    void IGCGFreeDTexture(IPObjectStruct *PObj) {}
    int IGCGFfdDraw(IPObjectStruct *PObj) { return FALSE; }
}

/*****************************************************************************
* DESCRIPTION:                                                               *
*   Redirects stdin/out/err to a second console window.                      *
*                                                                            *
* PARAMETERS:                                                                *
*   None                                                                     *
*                                                                            *
* RETURN VALUE:                                                              *
*   void                                                                     *
*****************************************************************************/
static void RedirectIOToConsole(void)
{
    int hConHandle;
    long lStdHandle;
    CONSOLE_SCREEN_BUFFER_INFO coninfo;
    FILE *fp;

    /* Allocate a console for this app. */
    AllocConsole();

    /* Set the screen buffer to be big enough to let us scroll text. */
    GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), 
			       &coninfo);
    coninfo.dwSize.Y = 1000;
    SetConsoleScreenBufferSize(GetStdHandle(STD_OUTPUT_HANDLE), 
			       coninfo.dwSize);

    /* Redirect unbuffered STDOUT to the console. */
    lStdHandle = (long) GetStdHandle(STD_OUTPUT_HANDLE);
    hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
    fp = _fdopen(hConHandle, "w");
    *stdout = *fp;
    setvbuf(stdout, NULL, _IONBF, 0);

    /* Redirect unbuffered STDIN to the console. */
    lStdHandle = (long) GetStdHandle(STD_INPUT_HANDLE);
    hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
    fp = _fdopen(hConHandle, "r");
    *stdin = *fp;
    setvbuf(stdin, NULL, _IONBF, 0);

    /* Redirect unbuffered STDERR to the console. */
    lStdHandle = (long) GetStdHandle(STD_ERROR_HANDLE);
    hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
    fp = _fdopen(hConHandle, "w");
    *stderr = *fp;
    setvbuf(stderr, NULL, _IONBF, 0);
}
