/******************************************************************************
* NCSimMain.h - Main header file of NCSim.				      *
*******************************************************************************
* (C) Gershon Elber					                      *
*******************************************************************************
* Written by Gershon Elber, Dec 2006.					      *
******************************************************************************/

#ifndef NCSIM_MAIN
#define NCSIM_MAIN

#include "wx/wx.h"
#include "wx/splitter.h"
#include "wx/dnd.h"
#include "wx/slider.h"
#include "wx/toolbar.h"
#include "wx/textctrl.h"
#include "wx/spinctrl.h"
#include "wx/aui/auibook.h"

#include "irit_sm.h"

#include "NCSimDefs.h"        /* Must be included first in all NCSim files. */
#include "NCSimulation.h"
#include "NCSimMessages.h"

#define NCSIM_MAX_NUM_RECENT_LOADED_FILES		5

#define  NCSimPrintf	NCSimMF -> Printf

enum {
    /* Menu events. */
    wxID_NCSIM_MENU_START = wxID_HIGHEST + 1000,

    wxID_NCSIM_FILE_MENU_IMPORT_GEOM = wxID_NCSIM_MENU_START,
    wxID_NCSIM_FILE_MENU_IMPORT_FINAL_PART,
    wxID_NCSIM_FILE_MENU_LOAD_NCFILE,
    wxID_NCSIM_FILE_MENU_SAVE_NCFILE,
    wxID_NCSIM_FILE_MENU_CLOSE_NCFILE,
    wxID_NCSIM_FILE_MENU_EXPORT_TPATH,
    wxID_NCSIM_FILE_MENU_SAVE_NCZBUF,
    wxID_NCSIM_FILE_MENU_EXIT,
    wxID_NCSIM_FILE_MENU_RECENT_FNAME,
    wxID_NCSIM_FILE_MENU_RECENT_FNAME_LAST = wxID_NCSIM_FILE_MENU_RECENT_FNAME
					   + NCSIM_MAX_NUM_RECENT_LOADED_FILES,

    wxID_NCSIM_STATE_MENU_SET_ALL_PARAMS,
    wxID_NCSIM_STATE_MENU_SET_OGL_PARAMS,

    wxID_NCSIM_VIEW_MENU_VIEW_UP,
    wxID_NCSIM_VIEW_MENU_VIEW_FRONT,
    wxID_NCSIM_VIEW_MENU_VIEW_SIDE,
    wxID_NCSIM_VIEW_MENU_RESCALE_VIEW,

    wxID_NCSIM_HELP_MENU_ABOUT,

    wxID_NCSIM_MENU_END = wxID_NCSIM_HELP_MENU_ABOUT,

    /* Animation key events. */
    wxID_NCSIM_ANIM_FBWRD,
    wxID_NCSIM_ANIM_BWRD,
    wxID_NCSIM_ANIM_BWRD_STEP,
    wxID_NCSIM_ANIM_STOP,
    wxID_NCSIM_ANIM_PAUSE,
    wxID_NCSIM_ANIM_FWRD_STEP,
    wxID_NCSIM_ANIM_FWRD,
    wxID_NCSIM_ANIM_FFWRD,
    wxID_NCSIM_ANIM_SLIDER,
    wxID_NCSIM_ANIM_LINE_NUM,
    wxID_NCSIM_ANIM_NC_TEXT,

    wxID_NCSIM_ANIM_LAST,

    /* Alloc SIM_INFO slots for simulation parameters widget. */
    wxID_NCSIM_PARAMS = wxID_NCSIM_ANIM_LAST + 1,
    wxID_NCSIM_PARAM_REL_ARC_CNTR = wxID_NCSIM_PARAMS,
    wxID_NCSIM_PARAM_REVERSE_Z_DIR,
    wxID_NCSIM_PARAM_TOOL_NUM,
    wxID_NCSIM_PARAM_UNITS,
    wxID_NCSIM_PARAM_MAX_MRR,
    wxID_NCSIM_PARAM_STOCK_GEOM_INIT,
    wxID_NCSIM_PARAM_TOOL_TYPE,
    wxID_NCSIM_PARAM_ZBUF_RES,
    wxID_NCSIM_PARAM_NC_Z_BUF,
    wxID_NCSIM_PARAM_PAUSE_ERROR,
    wxID_NCSIM_PARAM_UPDATE_FEEDRATES,
    wxID_NCSIM_PARAM_RESET_FEEDRATES,
    wxID_NCSIM_PARAM_SIM_MODE,
    wxID_NCSIM_PARAM_SIM_STEP,
    wxID_NCSIM_PARAM_FEED_RATE,
    wxID_NCSIM_PARAM_SPINDLE_SP,
    wxID_NCSIM_PARAM_FAST_SPEED_FACTOR,
    wxID_NCSIM_PARAM_GEOM_APPROX_TOL,
    wxID_NCSIM_PARAM_TOOL_DIAM,
    wxID_NCSIM_PARAM_TOOL_TORUS_RAD,
    wxID_NCSIM_PARAM_TOOL_ICOLOR,
    wxID_NCSIM_PARAM_TOOL_FCOLOR,
    wxID_NCSIM_PARAM_TOOL_CCOLOR,
    wxID_NCSIM_PARAM_STOCK_COLOR,
    wxID_NCSIM_PARAM_STOCK_TRANSP,
    wxID_NCSIM_PARAM_STOCK_BBOX,
    wxID_NCSIM_PARAM_OK,
    wxID_NCSIM_PARAM_SAVE,

    wxID_NCSIM_PARAM_LAST,
    wxID_NCSIM_PARAMS_SIZE = wxID_NCSIM_PARAM_LAST - wxID_NCSIM_PARAMS + 1,

    /* Alloc SIM_INFO slots for rendering (OGL) parameters widget. */
    wxID_NCSIM_OGL_PARAMS = wxID_NCSIM_PARAM_LAST + 2,
    wxID_NCSIM_OGL_PARAM_ORTHO_PERSP = wxID_NCSIM_OGL_PARAMS,
    wxID_NCSIM_OGL_PARAM_SOLID_WIRE,
    wxID_NCSIM_OGL_PARAM_LINE_WIDTH,
    wxID_NCSIM_OGL_PARAM_FRAME_SKIP,
    wxID_NCSIM_OGL_PARAM_MOUSE_SENS,

    wxID_NCSIM_OGL_PARAM_AXES_SIZE,
    wxID_NCSIM_OGL_PARAM_LIGHT0POS,
    wxID_NCSIM_OGL_PARAM_LIGHT0CLR,
    wxID_NCSIM_OGL_PARAM_LIGHT1POS,
    wxID_NCSIM_OGL_PARAM_LIGHT1CLR,

    wxID_NCSIM_OGL_PARAM_BG_COLOR,

    wxID_NCSIM_OGL_PARAM_DRAW_AXES,
    wxID_NCSIM_OGL_PARAM_DRAW_TOOL,
    wxID_NCSIM_OGL_PARAM_DRAW_TPATH,
    wxID_NCSIM_OGL_PARAM_DRAW_STOCK,
    wxID_NCSIM_OGL_PARAM_DRAW_FINAL_PART,
    wxID_NCSIM_OGL_PARAM_DRAW_OTHER_GEOM,
    wxID_NCSIM_OGL_PARAM_MOUSE_SENSITIVITY,

    wxID_NCSIM_OGL_PARAM_LAST,
    wxID_NCSIM_OGL_PARAMS_SIZE = wxID_NCSIM_OGL_PARAM_LAST - wxID_NCSIM_OGL_PARAMS + 2
};

/* The main NCSim wrapper. */
class NCSimApp : public wxApp
{
    public:
	virtual bool OnInit();    /* Called on application startup. */
};

/* Drag and drop class. */
class NCSimDnDFileClass : public wxFileDropTarget
{
    public:
	virtual bool OnDropFiles(wxCoord x, 
				 wxCoord y, 
				 const wxArrayString &Filenames);
};

/* The main split frame of NCSim. */
class NCSimSplitMainFrame : public wxSplitterWindow
{
    public:
	NCSimSplitMainFrame(wxFrame *Parent);

    private:
	DECLARE_EVENT_TABLE()
	DECLARE_NO_COPY_CLASS(NCSimSplitMainFrame)
};

/* The main frame of NCSim. */
class NCSimMainFrame : public wxFrame
{
    public:
	/* Con/Destructor. */
	NCSimMainFrame(const wxString& title, const wxSize &Size);
	~NCSimMainFrame();
	bool AddFName2RecentFNames(const char *RcntFileName);
	bool DeleteFName2RecentFNames(const char *RcntFileName);

	int UpdateFileMenu();
	int SaveGUIState();
	int LoadGUIState();

	/* Event handlers. */
	void OnQuit(wxCommandEvent &Event);
	void OnAbout(wxCommandEvent &Event);
	void OnRecentFNames(wxCommandEvent &Event);
	void NCSimPageChanged(wxAuiNotebookEvent &Event);
	void ImportGeometry(wxCommandEvent &Event);
	void ImportFinalPart(wxCommandEvent &Event);

	void LoadNCFile(wxCommandEvent &Event);
	void SaveNCFile(wxCommandEvent &Event);
	void CloseNCFile(wxCommandEvent &Event);
	void SaveNCTPathToFile(wxCommandEvent &Event);
	void SaveNCZBufToFile(wxCommandEvent &Event);

	void PopupMsg(const char *Msg,
		      NCSimLogStyleType Style = NCSIM_LOG_INFO);
	int PopupYesNo(const char *Msg,
		       NCSimLogStyleType Style = NCSIM_LOG_INFO);
	bool CreateNewSim(const char *FullPath);

        /* Controllers over the simulation progress toolbar. */
	void StateViewSimMode(wxCommandEvent &Event);
	void AnimControl(wxCommandEvent &Event);
	void ShowMenuHelp(wxMenuEvent &Event);
	void AnimSliderControl(wxCommandEvent &WXUNUSED(event));

	/* Controllers over the status bar. */
	void SetFPSNumPolysStatus(IrtRType FPS, int NumPolys);

#	ifdef USE_VARARGS
	    void Printf(NCSimLogStyleType Style, char *va_alist, ...);
#	else
	    void Printf(NCSimLogStyleType Style, char *Format, ...);
#	endif /* USE_VARARGS */

        NCSimulationClass *CrntSim;
  	wxAuiNotebook *TextNotebook;
	class NCSimOGLClass *OGLWin;
	bool UseOGLDispLists;

	wxStatusBar *StatusBar;
	wxSlider *AnimSlider;
	wxSpinCtrl *AnimNCLineNum;
	wxTextCtrl *AnimNCText;

    private:
	IritConfigStruct *BindSimGUIConfig(char **MainWindowPosStr,
					   char **MainWindowSizeStr,
					   char **WIndowsStateStr,
					   char ***RcntFileNamesStrs);
	void OnTextChange(wxCommandEvent &Event);

	char *RecentlyLoadedFiles[NCSIM_MAX_NUM_RECENT_LOADED_FILES];
	NCSimMessagesClass *LogWin;
	wxToolBar *TBar;
	wxPanel *PPanel;
	wxMenu *FileMenu;
	wxMenuBar *MenuBar;
	wxAuiManager *AuiManager;

        DECLARE_EVENT_TABLE()		      /* This class handles events. */
};

DECLARE_APP(NCSimApp)		  /* Implements NCSimApp& GetApp(). */

/* Tool tips info for everybody to use. */
const wxString NCSimToolTip(int Id);

/* This is our one and only main object. */
extern class NCSimMainFrame *NCSimMF;


inline const char *NCSimwxString2Chars(const wxString &Str) {
    return Str.mb_str(wxConvUTF8);
};

inline wxString NCSimChars2wxString(const char *String = "") {
    if (String == NULL)
	String = "";
    return wxString(String, wxConvUTF8, strlen(String));
};

#endif /* NCSIM_MAIN */
