/******************************************************************************
* NCSimOGL.h - Main entry point of OGL drawing class in NCSim.		      *
*******************************************************************************
* (C) Gershon Elber					                      *
*******************************************************************************
* Written by Gershon Elber, Dec 2006.					      *
******************************************************************************/

#ifndef NCSIM_OGL
#define NCSIM_OGL

#include "wx/glcanvas.h"
#include <wx/stattext.h>
#include <wx/spinctrl.h>
#include <wx/textctrl.h>
#include <wx/colordlg.h>
#include <wx/button.h>

#include "grap_lib.h"

#include "NCSimDefs.h"        /* Must be included first in all NCSim files. */
#include "OpenGL.h"

#define NCSIM_MAX_ZBUF_DISP_LIST	10

enum {
    MOUSE_KEY_NONE,
    MOUSE_LEFT_DOWN,
    MOUSE_MIDDLE_DOWN,
    MOUSE_RIGHT_DOWN
};

class NCSimOGLParamClass : public wxDialog
{
    friend class NCSimOGLClass;
    public:
        NCSimOGLParamClass(NCSimOGLClass *NCSimOGL,
			   wxWindow *Parent,
			   wxWindowID Id,
			   const wxString &Title = wxT(""),
			   int Flags = 0);
	void UpdateLightSource(int Index);
	void OnCheckBox(wxCommandEvent &Event);
	void OnTextChange(wxCommandEvent &Event);
	void OnComboBox(wxCommandEvent &Event);
	void OnButton(wxCommandEvent &Event);

	wxButton *SimModeBtn;

    private:
	class NCSimOGLClass *ThisNCSimOGL;

        /* Light sources params. */
	IrtRType Light0Pos[4], Light1Pos[4];
	int Light0Clr[3], Light1Clr[3];

        DECLARE_EVENT_TABLE()
};

class NCSimOGLClass: public wxGLCanvas
{
    friend NCSimOGLParamClass;
    public:
	/* Con/Desstructor. */
        NCSimOGLClass(wxWindow *Parent,
		      wxWindowID Id = wxID_ANY,
		      const wxPoint &Pos = wxDefaultPosition,
		      const wxSize &Size = wxDefaultSize,
		      long Style = 0,
		      const wxString &Name = _T("NCSimOGLClass") );
	NCSimOGLClass(wxWindow *Parent,
		      const NCSimOGLClass *other,
		      wxWindowID Id = wxID_ANY,
		      const wxPoint &Pos = wxDefaultPosition,
		      const wxSize &Size = wxDefaultSize,
		      long Style = 0,
		      const wxString &Name = _T("NCSimOGLClass") );

	~NCSimOGLClass();
	void SetOGLParams();
	void NCSimOGLInitSim();
	void NCSimPageChange();

	int SkipFrame() {
	    if (--SkippedFrames <= 0) {
	        SkippedFrames = SkipFrameRate;
		return FALSE;
	    }
	    else
	        return TRUE;
	}

	void CleanOGLObject(IPObjectStruct *PObj);
	void CleanOGLZBuf();

	void OnPaint(wxPaintEvent &Event);
	void OnSize(wxSizeEvent &Event);
	void OnEraseBackground(wxEraseEvent &Event);
	void OnKeyDown(wxKeyEvent &Event);
	void OnKeyUp(wxKeyEvent &Event);
	void OnMouseEvent(wxMouseEvent &Event);

	NCSimOGLParamClass *OGLParams;

    private:
	void InitGL();
	void RenderAxes();
	void RenderOneObject(IPObjectStruct *PObj,
			     int OGLDispListIndex);
	void RenderStockBottomWalls(bool DrawSides = FALSE);
	void RenderGeometry(IrtHmgnMatType Mat);
	void RenderCrntSim();
	void RenderOneRectangle(IrtPtType Pt1,
				IrtPtType Pt2,
				IrtPtType Pt3,
				IrtPtType Pt4,
				IrtPtType N);
	void PickGeometry(int x, int y);
        void Screen2ObjSpcCoords(int Sx,
				 int Sy,
				 IrtRType *Ix,
				 IrtRType *Iy);

	static bool GlblOGLInitialized;

	int ScrnViewportBias, ScrnWidth, ScrnHeight;

	long LastKey;
        int MouseX, MouseY, MouseKey;
	bool ForceDispListBuild;
        IrtRType AxesSize;

	/* Drawing Control parameters */
	bool DrawAxes, DrawTool, DrawToolPath, DrawStock, DrawFinalPart,
	     DrawOtherGeom;
	int SkipFrameRate, SkippedFrames;

        /* Display list params. */
	GLuint OGLDispListBase;
        int ZBufDispListActive[NCSIM_MAX_ZBUF_DISP_LIST]
			      [NCSIM_MAX_ZBUF_DISP_LIST];

	IGGraphicEventType LastEvent;

	wxGLContext *Context;

	DECLARE_EVENT_TABLE()		      /* This class handles events. */
};

#endif /* NCSIM_OGL */
