/******************************************************************************
* NCSimZBuffer.h - Zbuffer simulation core.				      *
*******************************************************************************
* (C) Gershon Elber					                      *
*******************************************************************************
* Written by Gershon Elber, Dec 2006.					      *
******************************************************************************/

#ifndef NCSIM_Z_BUFFER
#define NCSIM_Z_BUFFER

#include "irit_sm.h"
#include "iritprsr.h"
#include "rndr_lib.h"

#include "NCSimDefs.h"        /* Must be included first in all NCSim files. */

enum {
    NCSIM_ZBUF_FRONT_FACE = 1,
    NCSIM_ZBUF_BACK_FACE = 2
};

typedef enum {
    NCSIM_STOCK_GEOM_INIT_NONE,
    NCSIM_STOCK_GEOM_INIT_PLANE,
    NCSIM_STOCK_GEOM_INIT_SPHERE_CAP,
    NCSIM_STOCK_GEOM_INIT_SPHERE_CAVITY,
    NCSIM_STOCK_GEOM_INIT_CYLIN_CAP,
    NCSIM_STOCK_GEOM_INIT_CYLIN_CAVITY,
    NCSIM_STOCK_GEOM_INIT_GEN_SRF
} NCSimStockGeomInitType;

typedef void (*NCSimZBufTriangleCBFuncType)(IrtRType X1,
					    IrtRType Y1,
					    IrtRType Z1,
					    IrtRType X2,
					    IrtRType Y2,
					    IrtRType Z2,
					    IrtRType X3,
					    IrtRType Y3,
					    IrtRType Z3);

typedef void (*NCSimZBufQuadStripCBFuncType)(IrtRType X1,
					     IrtRType Y1,
					     IrtRType Z1,
					     IrtRType X2,
					     IrtRType Y2,
					     IrtRType Z2,
					     int LastQuad);

struct NCSimStockPlaneStruct {
    IrtRType ABCD[4];
};
struct NCSimStockSphereStruct {
    IrtPtType Cntr;
    IrtRType Rad;
};
struct NCSimStockCylinStruct {
    IrtPtType Pt1, Pt2;
    IrtRType Rad;
};
struct NCSimStockGenSrfStruct {
    IPObjectStruct *Geom;
    char *FName;
};
struct NCSimStockGeomInitStruct {
    NCSimStockGeomInitType Type;
    union {
        NCSimStockPlaneStruct Plane;
        NCSimStockSphereStruct Sphere;
        NCSimStockCylinStruct Cylin;
        NCSimStockGenSrfStruct GenSrf;
    };
};

class NCSimZBufferClass {
    public:
	/* Con/Desstructor. */
	NCSimZBufferClass(class NCSimulationClass *ThisSim,
			  int ZBufferSize = 1000,
			  GMBBBboxStruct *StockBoundingBox = NULL);
	~NCSimZBufferClass();

	bool ReCreateStockGeometry();
	void UpdateObj2ScreenMap();
	bool ZBufferInit(int GridSize,
			 class NCSimToolClass *Tools,
			 CagdBType UniformAspectRatio = TRUE);
	void ZBufferInitStockGeom();
	void ZBufferInitToolMask(NCSimToolClass *Tool);
	bool ZBufferRenderNCTool(NCSimToolClass *Tool, IrtPtType ToolPos);
        bool ZBufferRenderObj(IPObjectStruct *PObj,
			      IrtPtType Pos = NULL,
			      int DrawFrontBackFaces = (NCSIM_ZBUF_FRONT_FACE |
							NCSIM_ZBUF_BACK_FACE));
	void ZBufferGetGridSize(int *GridSizeX,
				int *GridSizeY,
				int *GridCellSizeX,
				int *GridCellSizeY);
	bool ZBufferGetCellZVals(int GridX,
				 int GridY,
				 IrtRType *ZValues,
				 int *XMin,
				 int *YMin,
				 int *XMax,
				 int *YMax);
	int ZBufferGetCellTris(int GridX,
			       int GridY,
			       NCSimZBufTriangleCBFuncType TriFunc);
	int ZBufferGetCellQuadStrip(int GridX,
				    int GridY,
				    NCSimZBufQuadStripCBFuncType QSFunc);
	void ZBufferUpdateCellsToRedraw(bool Reset = false);
	bool ZBufferGetActiveCells(int *XMin,
				   int *YMin,
				   int *XMax,
				   int *YMax);

	void UpdateObjGeometry(IPObjectStruct *PObjs,
			       NCSimObjectType ObjType,
			       bool Append = false);

        /* Stock info. */
	GMBBBboxStruct StockBBox;      /* Also serves to bound the ZBuffer. */
	int StockColor[3];
	IrtRType StockTransp;

	IPObjectStruct *PObjGnrl;       /* Possible other general geometry. */
        IPObjectStruct *FinalPartGeom;		    /* Final part geometry. */
	IPObjectStruct *StockWireGeom;      /* Wireframe rep. of the stock. */

	int RedrawCellXMin, RedrawCellXMax, RedrawCellYMin, RedrawCellYMax;

	int ZBufSize;

        /* Special stock geometry to initialize the Zbuffer with. */
	NCSimStockGeomInitStruct StockGeomInit;

        IrtRType GeomApproxTolerance;     /* Tool approximation tolerances. */

    private:
	class NCSimulationClass *TheSim;

        /* ZBuffer info. */
	INCZBufferPtrType ZBuf;
        int ZBufXSize, ZBufYSize;

	/* Screen <-> object space mapping values. */
	IrtPtType Obj2ScreenScale, Obj2ScreenTrans;

        /* These are used to estimate material removal rates. */
        IrtRType AccumulatedVolume, AccumulatedTime;
	int AccumulatedValidStep;
};

#endif /* NCSIM_Z_BUFFER */
