/******************************************************************************
* RealSpinCtrl.cpp - Holds numeric entry control.			      *
* This class implements spin control for real values.			      *
*******************************************************************************
* (C) Gershon Elber					                      *
*******************************************************************************
* Written by Gershon Elber, Dec 2006.					      *
******************************************************************************/

#ifndef REAL_SPIN_CTRL_H
#define REAL_SPIN_CTRL_H

#include <wx/textctrl.h>

#define REAL_SPIN_INFINITY	1e30
#define REAL_SPIN_COLOR_NORMAL		wxColour(0, 0, 130)
#define REAL_SPIN_COLOR_HIGHLIGHT	wxColour(160, 0, 0)

typedef void (*RealSpinCtrlUpdateValFunc)(double Val, wxWindowID Id);

class RealSpinCtrlClass : public wxTextCtrl
{
    public:
	RealSpinCtrlClass(double *ValueAddr,
			  double MinAllowedValue = -REAL_SPIN_INFINITY,
			  double MaxAllowedValue = REAL_SPIN_INFINITY,
			  double HIncrement = 0.0,
			  double VIncrement = 0.0,
			  wxWindow *Parent = NULL,
			  wxWindowID Id = wxID_ANY,
			  const wxPoint &Pos = wxDefaultPosition,
			  const wxSize &Size = wxDefaultSize,
			  int Flags = 0);
	RealSpinCtrlClass(double Value,
			  RealSpinCtrlUpdateValFunc UpdateValueFunc,
			  double MinAllowedValue = -REAL_SPIN_INFINITY,
			  double MaxAllowedValue = REAL_SPIN_INFINITY,
			  double HIncrement = 0.0,
			  double VIncrement = 0.0,
			  wxWindow *Parent = NULL,
			  wxWindowID Id = wxID_ANY,
			  const wxPoint &Pos = wxDefaultPosition,
			  const wxSize &Size = wxDefaultSize,
			  int Flags = 0);
	~RealSpinCtrlClass();
	void Refresh();
	void SetRealValue(char *NewVal);
	void SetRealValue(double NewVal);
	double GetRealValue();
	RealSpinCtrlUpdateValFunc SetUpdateFunction(RealSpinCtrlUpdateValFunc
								UpdateFunc) {
	    RealSpinCtrlUpdateValFunc
	        OldFunc = UpdateValFunc;

	    UpdateValFunc = UpdateFunc;
	    return OldFunc;
	}

    private:
	double MinAllowedVal, MaxAllowedVal, Val, *ValAddr, VInc, HInc;
	int MouseLastX, MouseLastY, MouseCaptured;
	wxWindowID WinId;
        RealSpinCtrlUpdateValFunc UpdateValFunc;

	void MarkTextValid(int Valid);
	void OnTextChange(wxCommandEvent &Event);
	void OnMouseEvent(wxMouseEvent &Event);
	void OnMouseMotionEvent(wxCommandEvent &Event);

        DECLARE_EVENT_TABLE()
};

#endif /* REAL_SPIN_CTRL_H */

